
;**********************************************************************
;    FILENAME:	9834_FreqRef.ASM
;    DATE:		DECEMBER 31, 2010
;    VERSION:	FINI
;    AUTHOR:	LARRY FOLTZER
;**********************************************************************
;
;    FILES REQUIRED: P16F627A.INC
;
;**********************************************************************
;   NOTES:
;	THIS CODE IS USED TO IMPLEMENT A REFERENCE FREQUENCY GENERATOR
;	DDS DEVICE IS ANALOG DEVICES AD9834 DDS IC
;
;**********************************************************************

	LIST      P=16F627A           ; LIST DIRECTIVE TO DEFINE PROCESSOR
	#INCLUDE <P16F627A.INC>       ; PROCESSOR SPECIFIC VARIABLE DEFINITIONS

	ERRORLEVEL  -302              ; SUPPRESS MESSAGE 302 FROM LIST FILE

	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT 

;		RAM ALLOCATION
ADDR_BUFFER		EQU	0X70
DATA_BUFFER		EQU	0X71
;
W_TEMP		EQU    	0X7E	; VARIABLE USED FOR CONTEXT SAVING 
STATUS_TEMP		EQU     	0X7F	; VARIABLE USED FOR CONTEXT SAVING

		ORG	0X020
FLAGS		RES	1
SHFT_FLAG		EQU	0	; B0 = 1 MEANS SHIFT KEY PRECEEDED COMMAND/SHIFT KEY FOLLOWED NUMBER INPUT
NO_NUM		EQU	7	; B7 = 0 MEANS NO NUMBER HAS YET BEEN RECEIVED

BIT_COUNT		RES	1
BYTE_CNT		RES	1
KEYCODE		RES	1
SR		RES	5
SUM		RES	4
VAL		RES	4

;		CONSTANT DEFINITIONS
EEPROM_BASE_ADDR_F0	EQU	0X10
EEPROM_BASE_ADDR_F1	EQU	0X18
EEPROM_BASE_ADDR_P0	EQU	0X20
EEPROM_BASE_ADDR_P1	EQU	0X28
;
;		HARDWARE DEFINITIONS FOR MCU 16F627A
;		DATA DIRECTION REGISTER PORT DEFINITIONS
;			TRISA	=	0110 0001	=	0X61
;			TRISB	=	0011 0000	=	0X30	0X00
;		INITIAL PORT OUTPUT VALUES
;			PORTA	=	1--0 000-	;	0X80	0X02
;			PORTB	=	01-- 0000	;	0X40	0X05
;	PORTA KEYPAD I/O
DAV	EQU	0
ACK	EQU	1
SDATA	EQU	6

;	PORTB I/O
DDS_FRAME	EQU	0	; MCU OUTPUT
DDS_DATA	EQU	1	; MCU OUTPUT
DDS_CLK	EQU	2	; MCU OUTPUT

;***************************
;	AD9834 MAX CLOCK FREQUENCY IS 75MHZ, AND HAS AN 28-BIT CORE.
;	WILL OPERATE AD9834 AT ((2^28)/4)HZ = 67.108864MHZ = 1/14.901NS
;	TO GENERATE 1MHZ OUTPUT (PERIOD=1US) MEANS WE MUST TRAVERSE
;	PHASE-AMPLITUDE TABLE IN (1US)/(14.9NS) ~67.11 STEPS OF SIZE
;	(2^28)/67.11 = 4E6. THEREFORE THE PHASE STEP INCREMENT FOR THE
;	DESIRED OUTPUT FREQUENCY IS 4 TIMES DESIRED FREQUENCY VALUE.
;
;	DEFAULT 1ST TIME STARTUP FREQUENCIES AND PHASES
;	4.9152MHZ; => (0X01 2C 00 00) X 4 = 0X04 B0 00 00 => F0-REG
;	4.0960MHZ; => (0X00 3E 80 00) X 4 = 0X00 FA 00 00 => F1-REG
;	INIT P0 IS 0 DEGREES, I.E., 0XC000	P0 REGISTER ADDRESS: B15:B13 = 110
;	INIT P1 TO 90 DEGREES, I.E., 0XE400	P1 REGISTER ADDRESS: B15:B13 = 111

;***************************
; EEPROM MEMORY USEAGE

; FREQUENCY REGISTER "F0"	FOUR (4) BYTES	MSB IN LOW ADDR:	0X10:0X13
; FREQUENCY REGISTER "F1"	FOUR (4) BYTES	MSB IN LOW ADDR:	0X18:0X1B

; FREQUENCY REGISTER "P0"	FOUR (4) BYTES	MSB IN LOW ADDR:	0X20:0X23
; FREQUENCY REGISTER "P1"	FOUR (4) BYTES	MSB IN LOW ADDR:	0X28:0X2B

; PRELOAD REGISTERS WITH 1ST TIME STARTUP VALUES
	ORG	0X2100
	DE	"Lawrence Foltzer"

	ORG	0X2110
	DE	0X00,0X3E,0X80,0X00	;	F0	4.0960MHZ; => (0X00 3E 80 00)

	ORG	0X2118
	DE	0X00,0X4B,0X00,0X00	;	F1	4.9152MHZ; => (0X00 4B 00 00)

	ORG	0X2120
	DE	0X00,0X00,0XC0,0X00	;	P0

	ORG	0X2128
	DE	0X00,0X00,0XE4,0X00	;	P1	90/360 = 1024/4096
;
;	(360/4096) X (DEGREES/STEP) X (PI/180) X (RADIANS/DEGREE) = 1.534 MRAD/STEP
;
;***************************
	ORG     	0X000		; PROCESSOR RESET VECTOR
	GOTO   	 INIT_CPU	; GO TO BEGINNING OF PROGRAM

	ORG    	 0X004		; INTERRUPT VECTOR LOCATION
	GOTO	INIT_CPU

;***************************
CMMD_KEY
	ANDLW	0X07
	BTFSS	FLAGS,NO_NUM	; WHEN NO_NUM = 0, NO NUMERIC INPUT BEFORE CMMD
	GOTO	PARAM_SELECT	; CMMD W/ NO NUMERIC. IF NO SHFT TOGGLE FREQ OR PHASE
; NUM:CMMD
	ADDWF	PCL,F
	GOTO	SHIFT
	GOTO	INIT_CPU
	GOTO	PROCESS_F0	; ENT
	GOTO	PROCESS_F1	; DEL
	GOTO	PROCESS_P0	; TKO
	GOTO	PROCESS_P1	; ABT
	GOTO	INIT_CPU
	GOTO	INIT_CPU

;***************************
;	KEYPAD MAP AND LABELING
;	0	1	2	3
;	4	5	6	7
;	8	9	SEQ	RST
;	ENT	DEL	TKO	ABT
;
;	SEQ = 0 = FUNCTION SHIFT KEY		RST = 1 = RESET
;	ENT = 2 = FREQUENCY_0			DEL = 3 = FREQUENCY_1
;	TKO = 4 = PHASE_0			ABT = 5 = PHASE_1

;***************************
PARAM_SELECT		; GOT HERE BECAUSE NO PRECEEDING NUMERIC INPUT
	ADDWF	PCL,F	; ISSUE COMMANDS TO CHANGE FREQ OR PHASE
	GOTO	INIT_CPU
	GOTO	INIT_CPU
	GOTO	F0_ON
	GOTO	F1_ON
	GOTO	P0_ON
	GOTO	P1_ON
	GOTO	INIT_CPU
	GOTO	INIT_CPU

;***************************
INIT_CPU
;	INIT CPU HW
	MOVLW	0X07
	MOVWF	CMCON
	MOVLW	B'11110011'
	MOVWF	PORTA
	MOVLW	B'11001000'
	MOVWF	PORTB

	BSF	STATUS,RP0
	CLRF	OPTION_REG
	MOVLW	B'01110001'
	MOVWF	TRISA
	MOVLW	B'00000000'
	MOVWF	TRISB
	BCF	STATUS,RP0
	CLRF	FLAGS

;***************************
INIT_DDS	; CONTROL REGISTER
;	START WITH CONTROL = 0X2110
;	REGISTER RESET STATE
;	CONTROL WITH REGISTER BITS
	MOVLW	0X23
	MOVWF	SR+2
	MOVLW	0X10
	MOVWF	SR+3
	CALL	SEND_16_BITS

;***************************
;	MOVE EEPROM DATA TO RAM, THEN TO AD9834
	MOVLW	SUM
	MOVWF	FSR		; AS POINTER
	MOVLW	EEPROM_BASE_ADDR_F0
	CALL	GET_PARAM
	CALL	SCALE_FREQ
	CALL	FORMAT_FREQ
;	ADD ADDRESS BITS
	MOVLW	0X40
	IORWF	SR,F
	IORWF	SR+2,F
	CALL	SEND_32_DDS_BITS

;***************************
	MOVLW	SUM
	MOVWF	FSR
	MOVLW	EEPROM_BASE_ADDR_F1
	CALL	GET_PARAM
	CALL	SCALE_FREQ
	CALL	FORMAT_FREQ
;	ADD ADDRESS BITS
	MOVLW	0X80
	IORWF	SR,F
	IORWF	SR+2,F
	CALL	SEND_32_DDS_BITS

;***************************
	MOVLW	SUM
	MOVWF	FSR
	MOVLW	EEPROM_BASE_ADDR_P0
	CALL	GET_PARAM
;	ADD ADDRESS BITS
	MOVF	SUM+2,W
	MOVWF	SR+2
	MOVF	SUM+3,W
	MOVWF	SR+3
	MOVLW	0X1F
	ANDWF	SR+2,F
	MOVLW	0XC0
	IORWF	SR+2,F
	CALL	SEND_16_BITS

;***************************
	MOVLW	SUM
	MOVWF	FSR
	MOVLW	EEPROM_BASE_ADDR_P1
	CALL	GET_PARAM
;	ADD ADDRESS BITS
	MOVF	SUM+2,W
	MOVWF	SR+2
	MOVF	SUM+3,W
	MOVWF	SR+3
	MOVLW	0X1F
	ANDWF	SR+2,F
	MOVLW	0XE0
	IORWF	SR+2,F
	CALL	SEND_16_BITS

;***************************
;	MAKE CONTROL = 0X2210
;	CLEAR RESET STATE
;	SHIFT CONTROL TO PORT PINS 
	MOVLW	0X22
	MOVWF	SR+2
	MOVLW	0X10
	MOVWF	SR+3
	CALL	SEND_16_BITS

;***************************
	BSF	PORTA,2
	BCF	PORTA,3
	GOTO	MAIN

;***************************
GET_PARAM
	MOVWF	ADDR_BUFFER
	MOVLW	.4
	MOVWF	BYTE_CNT
READ_LOOP
	MOVF	ADDR_BUFFER,W
	CALL	READEEM
	MOVWF	INDF
	INCF	FSR,F
	INCF	ADDR_BUFFER,F
	DECFSZ	BYTE_CNT,F
	GOTO	READ_LOOP
	RETURN
;***************************
F0_ON
	BCF	PORTB,5
	GOTO	EXIT
F1_ON
	BSF	PORTB,5
	GOTO	EXIT
P0_ON
	BCF	PORTB,4
	GOTO	EXIT
P1_ON
	BSF	PORTB,4
EXIT
	MOVLW	0X0C
	XORWF	PORTA,F
	CLRF	FLAGS
	GOTO	MAIN

;***************************
MAIN
	CLRF	SUM
	CLRF	SUM+1
	CLRF	SUM+2
	CLRF	SUM+3
	CLRF	VAL
	CLRF	VAL+1
	CLRF	VAL+2
	CLRF	VAL+3
W84DAVSET
	BTFSS	PORTA,DAV
	GOTO	W84DAVSET
	BSF	PORTA,ACK

;***************************
TEST4DAV
	BTFSC	PORTA,DAV
	GOTO	TEST4DAV

;***************************
;	WE NOW COLLECT KEYPAD BITS
GET_KEYPAD_BYTE
	CLRF	KEYCODE
	MOVLW	0X04
	MOVWF	BIT_COUNT
WATCH4BIT
	BTFSC	PORTA,DAV	;	DAV_INPUT
	GOTO	WATCH4BIT
	BSF	STATUS,C
	BTFSS	PORTA,SDATA	;	SDATA_INPUT
	BCF	STATUS,C
	RLF	KEYCODE,F
	BCF	PORTA,ACK	;	ACK_OUTPUT
W84NODAV
	BTFSS	PORTA,DAV	;	DAV_INPUT ON PA0
	GOTO	W84NODAV
	BSF	PORTA,ACK
	DECFSZ	BIT_COUNT,F
	GOTO	WATCH4BIT
	MOVLW	0X0A
	SUBWF	KEYCODE,W
	BTFSC	STATUS,C
	GOTO	CMMD_KEY

;***************************
NUM_KEY
;	MAKE HEX OF BASE 10 INPUT
;	DOUBLE CONTENTS OF "VAL" AND SAVE IN "SUM" AND "VAL"
	BSF	FLAGS,NO_NUM
	BCF	STATUS,C
	RLF	SUM+3,W	;	LSB
	MOVWF	SUM+3
	MOVWF	VAL+3
	RLF	SUM+2,W
	MOVWF	SUM+2
	MOVWF	VAL+2
	RLF	SUM+1,W
	MOVWF	SUM+1
	MOVWF	VAL+1
	RLF	SUM,W	;	MSB
	MOVWF	SUM
	MOVWF	VAL

;	CONTINUE SHIFTING "VAL" TO COMPUTE 8 TIMES INPUT "VAL"UE
	BCF	STATUS,C
	RLF	VAL+3,F
	RLF	VAL+2,F
	RLF	VAL+1,F
	RLF	VAL,F	;	4X
	BCF	STATUS,C
	RLF	VAL+3,F
	RLF	VAL+2,F
	RLF	VAL+1,F
	RLF	VAL,F	;	8X

;	ADD 2X (STORED IN "SUM") TO 8X (STORED IN "VAL"), LEAVING RESULT IN "SUM"
	MOVF	VAL+3,W
	ADDWF	SUM+3,F
	MOVF	VAL+2,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM+2,F
	MOVF	VAL+1,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM+1,F
	MOVF	VAL,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM,F

;	ADD KEYCODE TO SUM
	MOVF	KEYCODE,W
	ADDWF	SUM+3,F
	BTFSS	STATUS,C
	GOTO	GET_KEYPAD_BYTE
	MOVLW	.1
	ADDWF	SUM+2,F
	BTFSS	STATUS,C
	GOTO	GET_KEYPAD_BYTE
	ADDWF	SUM+1,F
	BTFSS	STATUS,C
	GOTO	GET_KEYPAD_BYTE
	ADDWF	SUM,F
	GOTO	GET_KEYPAD_BYTE
	
;***************************
SHIFT
	BSF	FLAGS,SHFT_FLAG
	GOTO	TEST4DAV

;***************************
SCALE_FREQ	; MULTIPLY BY 4
	BCF	STATUS,C
	RLF	SUM+3,F
	RLF	SUM+2,F
	RLF	SUM+1,F
	RLF	SUM,F
	BCF	STATUS,C
	RLF	SUM+3,F
	RLF	SUM+2,F
	RLF	SUM+1,F
	RLF	SUM,F
	RETURN

;***************************
FORMAT_FREQ
	MOVF	SUM+3,W	; LSB
	MOVWF	SR+1	; IS
	MOVF	SUM+2,W	; NOW
	ANDLW	0X3F	; READY
	MOVWF	SR	; FOR ADDR BITS

	MOVF	SUM,W
	MOVWF	SR+2
	MOVF	SUM+1,W
	MOVWF	SR+3
	MOVF	SUM+2,W
	MOVWF	SR+4	; DUMMY

	RLF	SR+4,F
	RLF	SR+3,F
	RLF	SR+2,F

	RLF	SR+4,F
	RLF	SR+3,F
	RLF	SR+2,W
	ANDLW	0X3F
	MOVWF	SR+2
; MSB READY FOR ADDRESS BITS
	RETURN

;***************************
SEND_32_DDS_BITS
	BCF	PORTB,DDS_FRAME
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SR0
	RLF	SR,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SR0
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SR1
	RLF	SR+1,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SR1
SEND_16_BITS
	BCF	PORTB,DDS_FRAME
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SR2
	RLF	SR+2,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SR2
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SR3
	RLF	SR+3,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SR3
	BSF	PORTB,DDS_FRAME
	RETURN

;***************************
; ENTER W/ ADDR IN "W", LEAVE W/ DATA IN "W"
READEEM
	BSF	STATUS,RP0
	MOVWF	EEADR
	BSF	EECON1,RD
	MOVF	EEDATA,W
	BCF	STATUS,RP0
	RETURN

;***************************
WRITEEEM
	BSF	STATUS,RP0
	MOVF	ADDR_BUFFER,W
	MOVWF	EEADR
	MOVF	DATA_BUFFER,W
	MOVWF	EEDATA
	BSF	EECON1,WREN
	MOVLW	0X55
	MOVWF	EECON2
	MOVLW	0XAA
	MOVWF	EECON2
	BSF	EECON1,WR
STILL_WORKING
	BTFSC	EECON1,WR
	GOTO	STILL_WORKING
	BCF	STATUS,RP0
	RETURN

;***************************
PROCESS_F0
	BTFSC	FLAGS,SHFT_FLAG
	CALL	PROG_F0
	CALL	SCALE_FREQ
	CALL	FORMAT_FREQ
	MOVLW	0X40
	IORWF	SR,F
	IORWF	SR+2,F
	CALL	SEND_32_DDS_BITS
	GOTO	F0_ON
;	GOTO	MAIN

;***************************
PROCESS_F1
	BTFSC	FLAGS,SHFT_FLAG
	CALL	PROG_F1
	CALL	SCALE_FREQ
	CALL	FORMAT_FREQ
	MOVLW	0X80
	IORWF	SR,F
	IORWF	SR+2,F
	CALL	SEND_32_DDS_BITS
	GOTO	F1_ON
;	GOTO	MAIN

;***************************
PROCESS_P0
	BTFSC	FLAGS,SHFT_FLAG
	CALL	PROG_P0
	MOVF	SUM+2,W
	MOVWF	SR+2
	MOVF	SUM+3,W
	MOVWF	SR+3
	MOVLW	0X1F
	ANDWF	SR+2,F
	MOVLW	0XC0
	IORWF	SR+2,F
	CALL	SEND_16_BITS
	GOTO	P0_ON
;	GOTO	MAIN

;***************************
PROCESS_P1
	BTFSC	FLAGS,SHFT_FLAG
	CALL	PROG_P1
	MOVF	SUM+2,W
	MOVWF	SR+2
	MOVF	SUM+3,W
	MOVWF	SR+3
	MOVLW	0X1F
	ANDWF	SR+2,F
	MOVLW	0XE0
	IORWF	SR+2,F
	CALL	SEND_16_BITS
	GOTO	P1_ON
;	GOTO	MAIN

;***************************
PROG_F0
; THE NEXT FOUR CODE GROUPS SIMPLY STORE KEYPAD INPUT
;  IN EEPROM IN UNSCALED, RAW HEX FORMAT.
	MOVLW	EEPROM_BASE_ADDR_F0
	MOVWF	ADDR_BUFFER
	MOVF	SUM,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+1,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+2,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+3,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	RETURN

;***************************
PROG_F1
	MOVLW	EEPROM_BASE_ADDR_F1
	MOVWF	ADDR_BUFFER
	MOVF	SUM,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+1,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+2,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+3,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	RETURN

;***************************
PROG_P0
	MOVLW	EEPROM_BASE_ADDR_P0+2
	MOVWF	ADDR_BUFFER
	MOVF	SUM+2,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+3,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	RETURN

;***************************
PROG_P1
	MOVLW	EEPROM_BASE_ADDR_P1+2
	MOVWF	ADDR_BUFFER
	MOVF	SUM+2,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	INCF	ADDR_BUFFER,F
	MOVF	SUM+3,W
	MOVWF	DATA_BUFFER
	CALL	WRITEEEM

	RETURN

;***************************
	END

