;***************************************;***************************************
;                                                                 
;    Author              				:  jeff bachiochi                             
;    Company             				:  imagine that 
;    Website            				:  www.imaginethatnow.com                               
;    Filename            				:  TSL1406R.s                
;    Date                				:  04/08/2011                      
;    File Version        				:  1.30                             
;                                                              
;    Other Files Required: p30F4012.gld, p30f4012.inc             
;    Tools Used:MPLAB GL : 7.01                               
;               Compiler : 1.30                               
;               Assembler: 1.30                                 
;               Linker   : 1.30                                 
;                                                               
;***************************************;***************************************
;    Notes:                                                                   
;    ======                                                                   
;                                                                             
;1. The device include file defines bit names and their location within SFR   
;   words. Replace the path to the include file, if necessary.                
;   For e.g., to use the p30f2010 device modify the .include directive to     
;   .include "Yourpath\p30F2010.inc"                                          
;                                                                             
;2. The "config" macro is present in the device specific '.inc' file and can  
;   be used to set configuration bits for the dsPIC 30F device.               
;   The '.inc' files also contain examples on using the "config" macro.       
;   Some examples have been provided in this file. The user application may   
;   require a different setting for the configuration bits.                   
;   The dsPIC 30F 16-bit MCU Family Refernce Manual(DS70046) explains the     
;   function of the configuration bits.                                       
;                                                                             
;3. The "__reset" label is the first label in the code section and must be    
;   declared global. The Stack Pointer (W15) and the Stack Pointer Limit      
;   register must be initialized to values that are past the run-time heap   
;   space, so as to avoid corrupting user data. Initializing these registers  
;   at the "__reset" label ensures that user data is not corrupted            
;   accidentally. The __SP_init and __SPLIM_init literals are defined by the  
;   linker. The linker automatically addresses above the run-time heap to     
;   these symbols. Users can change the __SP_init and __SPLIM_init literals to
;   suit their application.                                                   
;                                                                             
;4. A ".section <section_name>, <attributes>" directive declares a new section
;   named "<section_name>" with an attribute list that specifies if the       
;   section is located in Program Space(code), Uninitialized(bss) or          
;   Initialized Data Space(data). Refer to the document "dsPIC 30F Assembler, 
;   Linker and Utilities User's Guide" for further details.                   
;                                                                             
;5. Initialize W registers: Using uninitialized W registers as Effective      
;   Addresses(pointers) will cause an "Uninitialized W Register Trap" !       
;                                                                             
;6. The label "__T1Interrupt" is defined in the device linker script. The     
;   label defines the starting location in program space of the Timer1        
;   interrupt service routine (ISR).                                          
;   Similarly, the linker script defines labels for all ISRs. Notice, that    
;   the ISR label names are preceded by two underscore characters.            
;   When users needs to write ISR code, they must use these labels after      
;   providing them global scope. The linker will then place the ISR address in
;   the respective interrupt vector table location in program space.          
;   Context Save/Restore in ISRs can be performed using the double-word PUSH  
;   instruction,PUSH.D. User can also make use of MOV.D, PUSH and PUSH.S      
;   instructions. Refer dsPIC 30F 16-bit MCU Family Refernce Manual(DS70046)  
;   for further details.                                                      
;                                                                             
;***************************************;***************************************
;                                                                             
;   Additional Notes:                                                        
;                                                                             
;   TAOS TSL1406R is used to read the levels of light that
;	fall upon the sensor.  This sensor uses a 5KHz - 8MHz clock rate
;	for exposure control                                                                                                                                  
;       
;	AT Comands Include:
;
;	ATS			(do two output cycles)
;	ATD			(dump the data out the serial port)
;	ATDx		(dump data format 0=value as number of dots, 1=value as hex )
;	ATA			(do multiple two output cycles looking for an exposure where
;				all data falls within maximum and minimum parameters)
;	ATRC		(read integration count)
;	ATWCxxxx	(write integration count)
;	ATRX		(read maximum parameter)
;	ATWXxx		(write xx to maximum parameter)
;	ATRN		(read minimum parameter)
;	ATWNxx		(write xx to minimum parameter)
;                                                     
;***************************************;***************************************
;
        .equ __30F4012, 1
        .include "p30f4012.inc"
;                                                                             
;***************************************;***************************************
;
;	Configuration bits:
;                                                                             
;***************************************;***************************************
;

        config __FOSC, CSW_FSCM_OFF & XT_PLL16    ;Turn off clock switching and
                                        ;fail-safe clock monitoring and
                                        ;use the External Clock as the
                                        ;system clock

        config __FWDT, WDT_OFF          ;Turn off Watchdog Timer

        config __FBORPOR, PBOR_ON & BORV_27 & PWRT_16 & MCLR_EN
                                        ;Set Brown-out Reset voltage and
                                        ;and set Power-up Timer to 16msecs
                                            
        config __FGS, CODE_PROT_OFF     ;Set Code Protection Off for the 
                                        ;General Segment
        config __FICD, ICS_PGD			;PORTF debug and program
;                                                                             
;***************************************;***************************************
;
;	Program Specific Constants (literals used in code)
;                                                                             
;***************************************;***************************************
;
;***** PORTB
		.equ	VREF_P,	0				; PORTB.0
		.equ	VREF_M,	1				; PORTB.1
		.equ	AIN1,	2				; PORTB.2
		.equ	AIN0,	3				; PORTB.3
		.equ	CLK,	4				; PORTB.4
		.equ	SI,		5				; PORTB.5
;
;***** PORTC
		.equ	TX,		1				; PORTC.13
		.equ	RX,		1				; PORTC.14
;		.equ	15						; PORTC.15
;
;***** PORTD
;		.equ	0						; PORTD.0
;		.equ	1						; PORTD.1
;
;***** PORTE
		.equ	WaitToggle,	0			; PORTE.0
		.equ	UartBusy,	1			; PORTE.1
		.equ	LED4,	2				; PORTE.2
		.equ	LED3,	3				; PORTE.3
		.equ	LED2,	4				; PORTE.4
		.equ	LED1,	5				; PORTE.5
;
;
;		.equ	8						; PORTE.8					
;
;***** PORTF
;		.equ	2						; PORTF.2
;		.equ	3						; PORTF.3
;
;***** Flag
		.equ	Sample,	0				; 1=sample, 0=exposure
		.equ	RCR,	1				; 1=received a CR, 0=no CR 
		.equ	FC,		2				; 1=received first character,
										; 0=waiting on first character	
		.equ	NoRoom,	3				; 1=no room in buffer
		.equ	Val_Qty,4				; 1=Value, 0=Quantity (in dots)		
;
;***** EFLAG
		.equ	QMARK,	0				; 1=Error			
;
;***** Constants
		.equ	U1InputBufferLength,	0x20
		.equ	U1OutputBufferLength,	0x80
		.equ	SelectedDisplayDivisor,	0
		.equ	SelectedMaximumExposure,	0x66
		.equ	SelectedMinimumExposure,	0x04												
;                                                                             
;***************************************;***************************************
;
;	Global Declarations:
;                                                                             
;***************************************;***************************************
;
        .global _wreg_init       		; Provide global scope to _wreg_init 
                                 		; routine In order to call this routine 
                                 		; from a C file,place "wreg_init" in an 
                                 		; "extern" declaration in the C file.

        .global __reset          		; The label for the first line of code. 

        .global __T1Interrupt    		; Declare Timer 1 ISR name global
        .global __ADCInterrupt   		; Declare A/D ISR name global
        .global __U1TXInterrupt  		; Declare U1TX ISR name global
        .global __U1RXInterrupt  		; Declare U1RX ISR name global 
        .global __MathError    			; Declare Math Error name global 
        .global __AddressError    		; Declare Address Error name global   
        .global __StackError    		; Declare Stack Error name global 
        .global __OscillatorFail   		; Declare Oscillator Fail name global                                                      
;                                                                             
;***************************************;***************************************
;
;	Constants stored in Program space
;                                                                             
;***************************************;***************************************
;
        .section .usertables, code
        .text        
SignOnI:.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2
        .asciz	"Imagine That\r\n"
;        
SignOnF: .palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"FTB 254\r\n"
;        
SignOnT:.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"TSL1406R\r\n"
;        
AA:		.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Auto Exposure Routine\r\n"        
;        
TLL:	.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Too Little Light\r\n" 
;        
TML:	.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Too Much Light\r\n"         
;        
MinE:	.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Minimum Exposure\r\n"  
;        
MaxE:	.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Maximum Exposure\r\n" 
;        
NewE:	.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Lengthening Exposure\r\n"
;        
GE:		.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Good Exposure\r\n"  
;        
SC:		.palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2        
        .asciz	"Sampling Complete\r\n"                                                                              
;                                                                             
;***************************************;***************************************
;
;	Uninitialized variables in X-space in data memory
;                                                                             
;***************************************;***************************************
;
;         .section .xbss, bss, xmemory
;x_input: .space 2*SAMPLES        ;Allocating space (in bytes) to variable.
;                                                                             
;***************************************;***************************************
;
;	Uninitialized variables in Y-space in data memory
;                                                                             
;***************************************;***************************************
;
;          .section .ybss, bss, ymemory
;y_input: .space 2*SAMPLES        ;Allocating space (in bytes) to variable.
;                                                                             
;***************************************;***************************************
;
;	Uninitialized variables in Near data memory (Lower 8Kb of RAM)
;                                                                             
;***************************************;***************************************
;
        .section .nbss, bss, near
FLAG:	.space 2
EFLAG:	.space 2
Temp:	.space 2 
DisplayDivisor:	.space 2
MaximumExposure:	.space 2
MinimumExposure:	.space 2
CycleCount:	.space 2
ICHAR_RX: .space 2  
ICHAR_TX: .space 2
CHAR: .space 2
STATE: .space 2
VALUE: .space 2
MaxVALUE: .space 2
MinVALUE: .space 2
TARGET: .space 2
U1InputBufferHead:	.space	2
U1InputBufferTail:	.space	2
U1OutputBufferHead:	.space	2
U1OutputBufferTail:	.space	2
CountL: .space 2  						; CountL=1=64nS or clkper=4*64nS=256nS
										; N=384
										; Tint(min)=256nS*(N-18)+20uS = 113uS
										; Tint(max)=100mS
										; clkper=(100mS-20uS)/(N-18) = 99980uS/366 = 273uS
										; 273uS/4 = 91uS
										; 91uS/64nS = 1421 0r 0x5D8
Sample_1:  .space 384
Sample_385:  .space 384 
U1InputBufferBase:	.space U1InputBufferLength
U1OutputBufferBase:	.space U1OutputBufferLength
;                                                                             
;***************************************;***************************************
;
;	Code Section in Program Memory
;                                                                             
;***************************************;***************************************
;
.text                             		;Start of Code section
__reset:
        mov 	#__SP_init, W15    		;Initalize the Stack Pointer
        mov 	#__SPLIM_init, W0  		;Initialize the Stack Pointer Limit Register
        mov 	W0, 	SPLIM
        nop                       		;Add NOP to follow SPLIM initialization
        
        call 	_wreg_init         		;Call _wreg_init subroutine
                                  		;Optionally use RCALL instead of CALL
;
init_PORTB:
		mov		#0x000F,W0				; x15:2, AIN1=1, AIN0=1
		mov		W0,		PORTB		
		mov		#0x000F,W0				; x15:2, AIN1=I, AIN0=I
		mov		W0,		TRISB
;
init_PORTC:
		mov		#0x6000,W0				; x, TX=1, RX=1, x12:0
		mov		W0,		PORTC		
		mov		#0xBFFF,W0				; x, TX=O, RX=I, x12:0
		mov		W0,		TRISC
;
init_PORTD:
		mov		#0x0000,W0				; x15:0
		mov		W0,		PORTD		
		mov		#0xFFFF,W0				; x15:0
		mov		W0,		TRISD
;
init_PORTE:
		mov		#0x0000,W0				; x15:6, LED1:6=0
		mov		W0,		PORTE		
		mov		#0xFFFC,W0				; x15:6, LED1:6=O (LED outputs)
		mov		W0,		TRISE
;
init_PORTF:
		mov		#0x0003,W0				; x15:0
		mov		W0,		PORTF		
		mov		#0xFFFF,W0				; x15:0
		mov		W0,		TRISF
;
init_UART:
		mov		#95,	W0				; 95=19200 baud 15=115200 baud
		mov		W0,		U1BRG
		mov		#0x8420,W0				; 8,N,1
		mov		W0,		U1MODE
		mov		#0x8400,W0				; TXEN
		mov		W0,		U1STA				
;
init_AD:
		mov		#0x02E8,W0				; ADON=0, x, ADSIDL=0, x, x, x, FORM1:0=10, 
										; SSRC2:0=111, x, SIMSAM=1, ASAM=0, x, x
		mov		W0,		ADCON1
;
		mov		#0x6100,W0				; VCF2:0=011, x, x, CSCNA=0, CHPS1:0=01, 
										; BUFS=0, x, SMPI3:0=0000, BUFM=0, ALTS=0
		mov		W0,		ADCON2
;
		mov		#0x0103,W0				; x, x, x, SAMC4:0=00010,
										; ADRC=0, x, ADCS5:0=00000
		mov		W0,		ADCON3
;
		mov		#0x2222,W0				; CH123NB1:0=00, CH123SB=1, CH0NB=0, CH0SB3:0=0010,
										; CH123NA1:0=00, CH123SA=1, CH0NA=0, CH0SA3:0=0010
		mov		W0,		ADCHS		
;
		mov		#0xFFF0,W0				; PCFG15:4=1, PCFG3:0=0 
		mov		W0,		ADPCFG			; 1=digital, 0=analog
;
		mov		#0x0000,W0				; CSSL15:0=0 
		mov		W0,		ADCSSL			; 1=scan, 0=no scan	
;
		bset	ADCON1,	#ADON			; A/D running
		bclr	IFS0,	#ADIF			; clear interrupt
;
init_TIMER1:
		mov		#0x0030,W0				; TON=0, x, TSIDL=0, x12:8,  
										; x, TGATE=0, TCKPS1:0=11, x, TSYNC=0, TCS=0, x
		mov		W0,		T1CON
;		
		mov		#0xFFFF,W0				
		mov		W0,		PR1
;
		bset	T1CON,	#TON			; Timer running
		bclr	IFS0,	#T1IF			; clear interrupt
;
init_TIMER2:
		mov		#0x0000,W0				; TON=0, x, TSIDL=0, x12:8,  
										; x, TGATE=0, TCKPS1:0=00, x, TSYNC=0, TCS=0, x
		mov		W0,		T2CON
;		
		mov		#0x0001,W0				
		mov		W0,		PR1
;
		clr		TMR2					; Timer cleared
		bclr	IFS0,	#T1IF			; clear interrupt
;
init_VARIABLES:
		clr		CountL
		inc		CountL
		clr		FLAG
		clr		EFLAG
		mov		#SelectedDisplayDivisor,	W0
		mov		W0,		DisplayDivisor			
		mov		#SelectedMaximumExposure,	W0
		mov		W0,		MaximumExposure
		mov		#SelectedMinimumExposure,	W0
		mov		W0,		MinimumExposure		
		mov		#U1InputBufferBase,	W0
		mov		W0,		U1InputBufferHead
		mov		W0,		U1InputBufferTail
		mov		#U1OutputBufferBase,	W0
		mov		W0,		U1OutputBufferHead
		mov		W0,		U1OutputBufferTail				
;
init_INTERRUPTS:
;		bset	IEC0,	#U1TXIE			; enable interrupt
		bset	IEC0,	#U1RXIE			; enable interrupt
;		bset	IEC0,	#ADIE			; enable interrupt
		bset	IEC0,	#T1IE			; enable interrupt
;		bset	IEC0,	#T2IE			; enable interrupt	
;
		call	SignOn	
;
		call	SendOK
;		
		call	SendPrompt		
;                                                                             
;***************************************;***************************************
;
;	main prograqm loop
;                                                                             
;***************************************;***************************************
;
main:
		call	Look4Command
;		
        BRA     main              		;Last line of executed code
;
;***************************************;***************************************
;
;	Look For a Command
;
;***************************************;***************************************
;
Look4Command:
		btss	FLAG,	#RCR			; skip next if we need to do a command

		goto	Look4Command_Exit		; else nothing to do	
;
		call	UserInput
;
;***************************************
;
Look4AT:
		mov		#U1InputBufferBase,	W2		; start of buffer
;		
Look4AT_Chk4A:		
		mov		#'A',	W0
		clr		W1
		mov.b	[W2++],	W1
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
Look4AT_Chk4AT:		 
		mov		#'T',	W0
		mov.b	[W2++],	W1		
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
MightBeCommand:
;
;***************************************
;
Look4State3:
		mov		#0x03,	W0
		mov		STATE,	W1
		cpseq	W1,		W0				; skip next if State=3
		goto	Look4State4				; else look for State 4
;
State3:
;
State3_Chk4ATA:
		mov		#'A',	W0
		mov.b	[W2++],	W1		
		cpseq	W1,		W0
		goto	State3_Chk4ATD			; look for other commands
;
State3_ATA:
		mov 	#tblpage(AA), W0		; "Auto Exposure Routine"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(AA), W2
		call	Send2Uart
;
		clr		CountL
;		
State3_ATA_Loop:
		inc		CountL
		mov		#0x5D8,	W0
		mov		CountL,	W1
		cpsgt	W1,		W0				; skip next if CountL>0x5D8
		goto	State3_ATA_Exposure
;
		call	SendCR					; (uses W0, W1)
;		
		mov 	#tblpage(TLL), W0		; "Too Little Light"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(TLL), W2
		call	Send2Uart
;
		mov 	#tblpage(MaxE), W0		; "Maximum Exposure"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(MaxE), W2
		call	Send2Uart
;
		goto	State3_ATA_Exit			
;
State3_ATA_Exposure:
		bclr	FLAG,	#Sample
		call	Convert
;
		bset	FLAG,	#Sample		
		call	Convert
;		
		call	GetMaxMinValue
;
State3_ATA_Chk4MaxExposure:
		mov		MaximumExposure,	W0
		mov		MaxVALUE,	W1
		cpslt	W1,		W0				; skip next if MaxValue<MaximumExposure
		goto	State3_ATA_OptimumExposure
;
State3_ATA_Chk4MinExposure:
		mov		MinimumExposure,	W0
		mov		MinVALUE,	W1
		cpsgt	W1,		W0				; skip next if MinValue>MinimumExposure
		goto	State3_ATA_Continue
;
State3_ATA_OptimumExposure:
		mov		#1,		W0		
		mov		CountL,	W1
		cpseq	W1,		W0				; skip next if CountL=1
		goto	State3_ATA_Finish
;
		call	SendCR					; (uses W0, W1)			
;		
		mov 	#tblpage(TML), W0		; "Too Much Light"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(TML), W2
		call	Send2Uart
;
		mov 	#tblpage(MinE), W0		; "Minimum Exposure"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(MinE), W2
		call	Send2Uart
;
		goto	State3_ATA_Exit	
;
State3_ATA_Continue:
		mov		#1,		W0		
		mov		CountL,	W1
		cpseq	W1,		W0				; skip next if CountL=1
		goto	State3_ATA_Continue1
;		
		mov 	#tblpage(TLL), W0		; "Too Little Light"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(TLL), W2
		call	Send2Uart
;
		mov 	#tblpage(NewE), W0		; "Increasing Exposure"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(NewE), W2
		call	Send2Uart
;
State3_ATA_Continue1:
		mov		#'.',	W0				; <dot>
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output	; (uses W0, W1)
		goto	State3_ATA_Loop
;
State3_ATA_Finish:
		dec		CountL
		bclr	FLAG,	#Sample
		call	Convert
;
		bset	FLAG,	#Sample		
		call	Convert
;
		call	SendCR					; (uses W0, W1)		
;
		mov 	#tblpage(GE), W0		; "Good Exposure"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(GE), W2
		call	Send2Uart
;
		mov 	#tblpage(SC), W0		; "Sampling Complete"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(SC), W2
		call	Send2Uart		
;				
State3_ATA_Exit:		
		goto	Look4State_FinishedCommand
;
State3_Chk4ATD:
		mov		#'D',	W0
		cpseq	W1,		W0
		goto	State3_Chk4ATS			; look for other commands
;
State3_ATD:
		call	DisplayData				; (uses W0, W12, W13, W14)
;		
		goto	Look4State_FinishedCommand
;
State3_Chk4ATS:
		mov		#'S',	W0
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State3_ATS:
		bclr	FLAG,	#Sample
		call	Convert
;
		bset	FLAG,	#Sample		
		call	Convert
;
		mov 	#tblpage(SC), W0		; "Sampling Complete"
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(SC), W2
		call	Send2Uart
;				
		goto	Look4State_FinishedCommand											
;
;***************************************
;
Look4State4:
		mov		#0x04,	W0
		mov		STATE,	W1
		cpseq	W1,		W0				; skip next if State=4
		goto	Look4State6				; else look for State 6
;
State4:
;
State4_Chk4ATD:
		mov		#'D',	W0
		mov.b	[W2++],	W1		
		cpseq	W1,		W0
		goto	State4_Chk4ATR
;
State4_ATD:
;
State4_Chk4ATD0:
		mov		#'0',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	State4_Chk4ATD1
;
State4_ATD0:
		bclr	FLAG,	#Val_Qty		; format just quantity
		goto	State3_ATD
;
State4_Chk4ATD1:
		mov		#'1',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State4_ATD1:
		bset	FLAG,	#Val_Qty		; format just value
		goto	State3_ATD
;
State4_Chk4ATR:
		mov		#'R',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State4_ATR:
;
State4_Chk4ATRC:
		mov		#'C',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	State4_Chk4ATRN
;
State4_ATRC:
		mov		CountL,	W12
		call	SendWord				; (uses W0, W1)
;
		call	SendCR					; (uses W0, W1)
;	
		goto	Look4State_FinishedCommand
;
State4_Chk4ATRN:
		mov		#'N',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	State4_Chk4ATRX
;
State4_ATRN:
		mov		MinimumExposure,	W12
		call	SendByte				; (uses W0, W1)
;
		call	SendCR					; (uses W0, W1)
;	
		goto	Look4State_FinishedCommand
;
State4_Chk4ATRX:
		mov		#'X',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State4_ATRX:
		mov		MaximumExposure,	W12
		call	SendByte				; (uses W0, W1)
;
		call	SendCR					; (uses W0, W1)
;	
		goto	Look4State_FinishedCommand	
		;
;***************************************
;
Look4State6:
		mov		#0x06,	W0
		mov		STATE,	W1
		cpseq	W1,		W0				; skip next if State=6
		goto	Look4State8	
;
State6_Chk4ATW:
		mov		#'W',	W0
		mov.b	[W2++],	W1
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State6_ATW:
;
State6_Chk4ATWN:
		mov		#'N',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	State6_Chk4ATWX
;
State6_ATWN:
;
State6_Chk4ATWNhh:
		clr		TARGET
		mov.b	[++W2],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		push.d	W2
		mov		#0x10,W0
		mul		VALUE
		mov		W2,		W0
		add		TARGET
		pop.d	W2
;
		mov.b	[++W2],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		mov		VALUE,		W0
		add		TARGET
		mov		TARGET,		W1
		mov		W1,		MinimumExposure
		mov		W1,		W12
		call	SendByte				; (uses W0, W1)
;
		call	SendCR					; (uses W0, W1)	
;	
		goto	Look4State_FinishedCommand
;
State6_Chk4ATWX:
		mov		#'X',	W0
		mov.b	[W2],	W1		
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State6_ATWX:
;
State6_Chk4ATWXhh:
		clr		TARGET
		mov.b	[++W2],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		push.d	W2
		mov		#0x10,W0
		mul		VALUE
		mov		W2,		W0
		add		TARGET
		pop.d	W2
;
		mov.b	[++W2],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		mov		VALUE,		W0
		add		TARGET
		mov		TARGET,		W1
		mov		W1,		MaximumExposure
		mov		W1,		W12
		call	SendByte				; (uses W0, W1)
;
		call	SendCR					; (uses W0, W1)
;			
		goto	Look4State_FinishedCommand						
;
;***************************************
;
Look4State8:
		mov		#0x08,	W0
		mov		STATE,	W1
		cpseq	W1,		W0				; skip next if State=8
		goto	Look4StateEOL			; no such command	
;
State8_Chk4ATW:
		mov		#'W',	W0
		mov.b	[W2++],	W1
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State8_ATW:
;
State8_Chk4ATWC:
		mov		#'C',	W0
		mov.b	[W2++],	W1		
		cpseq	W1,		W0
		goto	Look4StateEOL			; no such command
;
State8_ATWC:
;
State8_Chk4ATWChhhh:
		mov.b	[W2++],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		push.d	W2
		mov		#0x1000,W0
		mul		VALUE
		mov		W2,		TARGET
		pop.d	W2
;
		mov.b	[W2++],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		push.d	W2
		mov		#0x100,W0
		mul		VALUE
		mov		W2,		W0
		add		TARGET
		pop.d	W2
;
		mov.b	[W2++],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		push.d	W2
		mov		#0x10,W0
		mul		VALUE
		mov		W2,		W0
		add		TARGET
		pop.d	W2
;
		mov.b	[W2++],	W1		
		mov		W1,		VALUE
		call	GetValue				; (uses W0, W1)
;
		btsc	VALUE,	#7				; skip next if MSBit=0
		goto	Look4StateEOL			; no such command		
;
		mov		VALUE,		W0
		add		TARGET
;
		mov		#0x1F5D,	W0
		mov		TARGET,		W1
		cpslt	W1,		W0				; skip next if TARGET<0x1F5D
		mov		W0,		W1				; else use max
		mov		W1,		CountL
		mov		W1,		W12
		call	SendWord				; (uses W0, W1)
;
		call	SendCR					; (uses W0, W1)	
;	
		goto	Look4State_FinishedCommand				
;
;***************************************
;
Look4StateEOL:							; we've reached the end of the line
		bclr	FLAG,	#RCR			; clear the flag indicating no command to do
		bset	EFLAG,	#QMARK
		call	SendErrors
;		
		goto	Look4Command_Exit	
;
Look4State_FinishedCommand:
		bclr	FLAG,	#RCR			; clear the flag indicating no command to do
		call	SendOK
;		
		call	SendPrompt
Look4Command_Exit:		
		return
;
;***************************************;***************************************
;
;	Get the maximum and minimum value of any exposure 1-768
;
;	uses W0, W1, W2, W10, W11
;
;	MaxVALUE 
;	MinVALUE
;
;***************************************;***************************************
;
GetMaxMinValue:
		clr		W1
		mov		#0x00FF,	W2
		clr		W10
		mov		#Sample_1,	W11
;		
GetMaxMinValue_Loop:
		clr		W0
		mov.b	[W11++],	W0
		cpsgt	W1,		W0		
		mov		W0,		W1
;		
		cpslt	W2,		W0
		mov		W0,		W2
;		
		inc		W10,	W10
		mov		#768,	W0	
		cpseq	W10,	W0
		goto	GetMaxMinValue_Loop
;		
		mov		W1,		MaxVALUE
		mov		W2,		MinVALUE
		return		
;
;***************************************;***************************************
;
;	Display Data
;
;	uses W0, W1, W10, W11, W12, W13
;
;***************************************;***************************************
;
DisplayData:
		clr		W10
		mov		#Sample_1,	W11
;
DisplayData_Loop:
		mov		W10,	W12
		call	SendWord				; (uses W0, W12, W13)
;
		mov		#0x20,	W0				; <sp>
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output	; (uses W0, W1)
;
		clr		W12
		mov.b	[W11++],W12		
		mov		DisplayDivisor,	W2
;		
		btss	FLAG,	#Val_Qty
		goto	DisplayData_Quantity
;
DisplayData_Value:
		call	SendByte
;
		mov		#0x20,	W0				; <sp>
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output	; (uses W0, W1)	
;
		goto	DisplayData_LineDone						
;
DisplayData_Quantity:
;		
DisplayData_FactorLoop:		
		cp0		W2
		bra		Z,		DisplayData_FactorDone
		lsr		W12,	W12
		dec		W2,		W2
		goto	DisplayData_FactorLoop

;
DisplayData_FactorDone:
		cp0		W12
		bra		Z,		DisplayData_LineDone
		mov		#'.',	W0				; <dot>
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output	; (uses W0, W1)	
;
		dec		W12,	W12
		goto	DisplayData_FactorDone		
;
DisplayData_LineDone:
		call	SendCR					; (uses W0, W1)
;
		inc		W10,	W10
		mov		#768,	W0	
		cpseq	W10,	W0
		goto	DisplayData_Loop
;
DisplayData_Exit:						
		return						
;
;***************************************;***************************************
;
;	Get binary value of ASCII number
;
;	VALUE (binary)	=	VALUE (ascii)
;
;	WREG	=	0-15=OK, 255=Error
;***************************************;***************************************
;
GetValue:	
		mov		#0x2F,	W0
		mov		VALUE,	W1					
		cpsgt	W1,		W0				; skip next if VALUE>ASCII(0)
		goto	GetValue_Error
;		
		mov		#0x39,	W0					
		cpsgt	W1,		W0				; skip next if VALUE>ASCII(9)
		goto	GetValue_0_9
;
		mov		#0x40,	W0					
		cpsgt	W1,		W0				; skip next if VALUE>ASCII(@)
		goto	GetValue_Error
;		
		mov		#0x46,	W0					
		cpsgt	W1,		W0				; skip next if VALUE>ASCII(F)
		goto	GetValue_A_F
;
GetValue_Error:
		setm	W0
		mov		W0,		VALUE		
		goto	GetValue_Exit							
;
GetValue_0_9:
		mov		#0x30,	W0	
		sub		W1,		W0,		W1		; VALUE=VALUE-0x30
		goto	GetValue_OK
;
GetValue_A_F:
		mov		#0x37,	W0	
		sub		W1,		W0,		W1		; VALUE=VALUE-0x37
;
GetValue_OK:
		mov		W1,		VALUE
;
GetValue_Exit:				
		return
;
;***************************************;***************************************
;
;	Get ASCII value of binary number
;
;	VALUE	=	binary
;	VALUE	=	ASCII
;
;***************************************;***************************************
;
GetASC:	
		mov		#0x09,	W0
		mov		VALUE,	W1					
		cpsgt	W1,		W0				; skip next if VALUE>9
		goto	GetASC_0_9				; else must be 0-9
;
GetASC_A_F:
		mov		#0x37,	W0	
		add		W0,		W1,		W1		; VALUE=VALUE+0x37
		goto	GetASC_Exit	
;		
GetASC_0_9:
		mov		#0x30,	W0	
		add		W0,		W1,		W1		; VALUE=VALUE+0x30
;
GetASC_Exit:
		mov		W1,		VALUE				
		return
;
;***************************************;***************************************
;
;	Send Word/Byte
;
;	uses W0, W12, W13
;
;***************************************;***************************************
;
SendWord:
		mov		#0x1000,	W13
		repeat	#17 					; Execute DIV.U 18 times		
		div.u	W12,	W13				; W0=quotient, W1=remainder
		add		#0x30,	W0				; '0x1000's'
		mov		#0x3A,	W12
		cpslt	W0,		W12				; skip next if W0<W12
		add		#7,		W0
		mov		W0,		ICHAR_TX
		mov		W1,		W12
		mov		#0x100,	W13		
		call	U1RXInterrupt_Output
;
		repeat	#17 					; Execute DIV.U 18 times		
		div.u	W12,	W13				; W0=quotient, W1=remainder
		add		#0x30,	W0				; '0x100's'
		mov		#0x3A,	W12
		cpslt	W0,		W12				; skip next if W0<W12
		add		#7,		W0
		mov		W0,		ICHAR_TX
		mov		W1,		W12
		call	U1RXInterrupt_Output
;
SendByte:
		mov		#0x10,	W13
		repeat	#17 					; Execute DIV.U 18 times		
		div.u	W12,	W13				; W0=quotient, W1=remainder
		add		#0x30,	W0				; '0x10's'
		mov		#0x3A,	W12
		cpslt	W0,		W12				; skip next if W0<W12
		add		#7,		W0	
		mov		W0,		ICHAR_TX
		mov		W1,		W12		
		call	U1RXInterrupt_Output
;	
		mov		W12,	W0	
		add		#0x30,	W0				; '1's'	
		mov		#0x3A,	W12
		cpslt	W0,		W12				; skip next if W0<W12
		add		#7,		W0	
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;
		return
;
;***************************************;***************************************
;
;	Send any Error Message
;
;***************************************;***************************************
;
SendErrors:
SendErrors_Chk4BFULL:
		btss	EFLAG,	#QMARK			; skip next if Buffer Full error
		goto	SendQuestionMark_Exit
;		
SendErrors_QMARK:		
		mov		#'?',	W0				; '?'
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;
		call	SendCR			
;		
		bclr	EFLAG,	#QMARK		
;
SendQuestionMark_Exit:
		return
;
;***************************************;***************************************
;
;	Send a 'OK'
;
;***************************************;***************************************
;
SendOK:
		mov		#'O',	W0				; 'O'
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;
		mov		#'K',	W0				; 'K'
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output		
;
		call	SendCR
;
SendOK_Exit:
		return		
;
;***************************************;***************************************
;
;	Send a '>'
;
;***************************************;***************************************
;
SendPrompt:
		mov		#'>',	W0				; '>'
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;
SendPrompt_Exit:
		return	
;
;***************************************;***************************************
;
;	Send a <cr> (and potential <lf>
;
;***************************************;***************************************
;
SendCR:
		mov		#0x0D,	W0				; <cr>
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;		
		mov		#0x0A,	W0				; <lf>
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output	
;
SendCR_Exit:
		return
;
;***************************************;***************************************
;
;	Check for CHAR in Input Buffer
;
;	CHAR	=	input character
;	STATE	=	number of characters
;
;***************************************;***************************************
;	
UserInput:								
		mov		#U1InputBufferBase,	W0
		mov		U1InputBufferHead,	W1
		cpseq	W1,		W0				; skip next if no data
		goto	UserInput_GotSome
;
UserInput_None:
		clr		STATE					; no characters
		goto	UserInput_Exit
;
UserInput_GotSome:
		sub		W1,		W0,		W0
		mov		W0,		STATE			; number of characters in buffer
		mov.b	[W1],	W0
		mov		W0,		CHAR			; move @Head to CHAR
;
UserInput_Exit:					
		return
;                                                                             
;***************************************;***************************************
;
;Subroutine: Initialization of W registers to 0x0000
;                                                                             
;***************************************;***************************************
;

_wreg_init:
        clr		W0
        mov 	W0, 	W14
        repeat 	#12
        mov 	W0, 	[++W14]
        clr 	W14
        return
;                                                                             
;***************************************;***************************************
;
;	UART1 TX Service Routine
;                                                                             
;***************************************;***************************************
;
__U1TXInterrupt: 
		push.s
;
U1TXInterrupt_Loop:		
		mov		U1OutputBufferTail,	W1
		mov		U1OutputBufferHead,	W0
		cpseq	W0,		W1				; skip if nothing to send
		goto	U1TXInterrupt_Character
;
U1TXInterrupt_Done:
		bclr	IEC0,	#U1TXIE			; no more characters, stop interrupts
		goto	U1TXInterrupt_Exit
;
U1TXInterrupt_Character:
		mov		#U1TXREG,	W0
		mov.b	[W1++],[W0]				; move @Tail to transmitter
;
		mov		#U1OutputBufferBase+#U1OutputBufferLength,	W0	; End Of Buffer
		cpseq	W1,		W0				; skip next if UARTOutputBufferTail=EOB	
		goto	U1TXInterrupt_TailOK	; no adjustment needed
;
U1TXInterrupt_AdjustTail:
		mov		#U1OutputBufferBase,	W1
;
U1TXInterrupt_TailOK:		
		mov		W1,		U1OutputBufferTail	; save the Tail pointer
		btsc	U1STA,	#UTXBF			; skip next if transmitter is not full
		goto	U1TXInterrupt_ClearInterrupt	; done for now	
;
U1TXInterrupt_TransmitterMT:
		goto	U1TXInterrupt_Loop		; try moving another		
;
U1TXInterrupt_ClearInterrupt:
		bclr	IFS0,	#U1TXIF			; clear interrupt
;
U1TXInterrupt_Exit:		
		pop.s
		retfie  
;                                                                             
;***************************************;***************************************
;
;	UART1 RX Service Routine
;                                                                             
;***************************************;***************************************
;
__U1RXInterrupt: 
		push.s	
		btsc	U1STA,	#FERR			; skip next if no framing error
		goto	U1RXInterrupt_FERR
;
		btsc	U1STA,	#OERR			; skip next if no overrun error
		goto	U1RXInterrupt_PERR
;
U1RXInterrupt_GoodCharacter:
		mov		U1RXREG,W0
		mov		W0,		ICHAR_RX 		; save the incoming character
;
U1RXInterrupt_Chk4Legal_CR:		
		mov		#0x0D,	W0				; <cr>
		mov		ICHAR_RX,	W1
		cpseq	W0,		W1				; skip next if ICHAR=<cr>
		goto	U1RXInterrupt_Chk4Legal_NUM	; not <cr>, keep checking	
;
U1RXInterrupt_CR:
		goto	U1RXInterrupt_Legal
;
U1RXInterrupt_Chk4Legal_NUM:		
		mov		#0x2F,	W0				; '/'
		cpsgt	W1,		W0  			; skip next if ICHAR>'/'
		goto	U1RXInterrupt_Exit		; control, leave without saving
;
U1RXInterrupt_Chk4Legal_NUM_1:		
		mov 	#0x3A,	W0				; ':'
		cpsgt	W1,		W0				; skip next if ICHAR>':'
		goto	U1RXInterrupt_Legal		; must be NUMERIC
;
U1RXInterrupt_Chk4Legal_CHAR:	
		bclr	ICHAR_RX,	#5			; make all upper case
		mov		ICHAR_RX,	W1
;			
		mov		#0x3F,	W0				; '?'
		cpsgt	W1,		W0				; skip next if ICHAR>'?'
		goto	U1RXInterrupt_Exit		; punctuation, leave without saving
;
U1RXInterrupt_Chk4Legal_CHAR_1:		
		mov		#0x5B,	W0 				; '['
		cpsgt	W1,		W0				; skip next if ICHAR>'['
		goto	U1RXInterrupt_Legal			
;
		goto	U1RXInterrupt_Exit		; punctuation, leave without saving	
;
U1RXInterrupt_Legal:
		mov		ICHAR_RX,	W0
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;
		mov		ICHAR_RX,	W1
		mov		#0x0D,	W0				; was that a <cr>?
		cpseq	W1,		W0
		goto	U1RXInterrupt_Legal_Chk4RCR
;		
		mov		W1,		ICHAR_TX		; <cr>
		call	U1RXInterrupt_Output
;
		mov		#0x0A,	W0
		mov		W0,		ICHAR_TX		; <lf>
		call	U1RXInterrupt_Output
;		
U1RXInterrupt_Legal_Chk4RCR:
		btsc	FLAG,	#RCR			; skip next if command has been processed
		goto	U1RXInterrupt_Exit		; else leave without saving character
;
		mov		ICHAR_RX,	W1
		mov		#0x0D,	W0				; <cr>
		cpseq	W1,		W0 				; skip next if this is a <cr>
		goto	U1RXInterrupt_Legal_Chk4FC		; else go on
;		
		goto	U1RXInterrupt_Legal_CR	; we need to do a command
;
U1RXInterrupt_Legal_Chk4FC:
		btsc	FLAG,	#FC				; skip next if the first character
		goto	U1RXInterrupt_Check4Room; else not the first, go check if buffer full
;
U1RXInterrupt_Legal_FC:
		bset	FLAG,	#FC				; set flag to indicate first character	
;
U1RXInterrupt_Legal_ClearBuffer:
		mov		#U1InputBufferBase,	W0
		mov		W0,		U1InputBufferHead
;
U1RXInterrupt_Check4Room:
		mov		U1InputBufferHead,	W0	; offset
		mov		#U1InputBufferBase+#U1InputBufferLength-1,	W2	; End Of Buffer
		cpseq	W2,		W0				; skip next if ITEMP=EOB
		goto	U1RXInterrupt_HeadInc	; else we have room, go save to buffer
;
U1RXInterrupt_NoRoom:
		goto	U1RXInterrupt_Legal_FC	; leave without saving character
;
U1RXInterrupt_Legal_CR:
		bset	FLAG,	#RCR			; set flag to indicate <cr>
		bclr	FLAG,	#FC				; clear flag to indicate looking for first character		
		goto	U1RXInterrupt_Exit		; leave without saving character		
;
U1RXInterrupt_HeadInc:
		mov.b	W1,		[W0++]			; move receiver to @Head
		mov		W0,	U1InputBufferHead
;
		goto	U1RXInterrupt_Exit		; done here 
;
U1RXInterrupt_PERR:
		bclr	U1STA,	#PERR			; clear the error
		goto	U1RXInterrupt_NoRoom	; and dump the character
;
U1RXInterrupt_FERR:
		bclr	U1STA,	#FERR			; clear the error
		goto	U1RXInterrupt_NoRoom	; and dump the character		
;
U1RXInterrupt_Exit:
		bclr	IFS0,	#U1RXIF
		pop.s
		retfie	
;                                                                             
;***************************************
;
U1RXInterrupt_Output:
		bset	PORTE,	#UartBusy			; external indication of this routine
		disi	#24
;		
U1RXInterrupt_Output_Check4Room:
		mov		U1OutputBufferHead,	W1
		inc		W1,		W1		
		mov		#U1OutputBufferBase+#U1OutputBufferLength,	W0	; End Of Buffer
		cpseq	W0,	W1					; skip next if TEMP=EOB
		goto	U1RXInterrupt_Output_HeadOK	; else no adjustment needed
;
U1RXInterrupt_Output_AdjustHead:
		mov		#U1OutputBufferBase,	W1
;
U1RXInterrupt_Output_HeadOK:
		mov		U1OutputBufferTail,	W0
		cpseq	W0,		W1				; skip next if no room
		goto	U1RXInterrupt_Output_Room
;
U1RXInterrupt_Output_NoRoom:
		disi	#0
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		goto	U1RXInterrupt_Output
;
U1RXInterrupt_Output_Room:
		mov		ICHAR_TX,	W0
		mov		U1OutputBufferHead,	W1		
		mov.b	W0,	[W1]				; move CHAR to @Head
		inc		W1,		W1				
;
		mov		#U1OutputBufferBase+#U1OutputBufferLength,	W0	; End Of Buffer
		cpseq	W0,	W1					; skip next if TEMP=EOB
		goto	U1RXInterrupt_Output_HeadOK1	; else no adjustment needed
;
U1RXInterrupt_Output_AdjustHead1:
		mov		#U1OutputBufferBase,	W1
;
U1RXInterrupt_Output_HeadOK1:		
		mov		W1,	U1OutputBufferHead	; save the Head pointer
		bset	IEC0,	#U1TXIE			; let the UART send any characters			 
		goto	U1RXInterrupt_Output_Exit	; and leave		
;
U1RXInterrupt_Output_Exit:
		bclr	PORTE,	#UartBusy			; external indication of this routine
		disi	#0
		return			     
;                                                                             
;***************************************;***************************************
;
;	A/D Interrupt Service Routine
;                                                                             
;***************************************;***************************************
;
__ADCInterrupt:
		bclr	IFS0,	#ADIF
		retfie
;                                                                             
;***************************************;***************************************
;
;	Timer 1 Interrupt Service Routine
;                                                                             
;***************************************;***************************************
;
__T1Interrupt:
        bclr 	IFS0, 	#T1IF           ; Clear the Timer1 Interrupt flag Status
                                 		; bit.
        btg		PORTE,	#LED1			; toggle LED1	                         		
        retfie                     		; Return from Interrupt Service routine
;                                                                             
;***************************************;***************************************
;
;	Convert								Go through a Convert cycle (385 clocks)
;										for exposure or conversion  
;
;	Flag.Sample	1=read last exposure, 0=take exposure
;                                                                             
;***************************************;***************************************
;
Convert:
		mov		#Sample_1,	W2
		mov		#Sample_385,	W3
		bset	PORTB,	#SI				; start cycle
		clr		CycleCount
		inc		CycleCount
;
		bset	PORTB,	#CLK			; clock high
;		
		mov		CountL,	W0
		mov		W0,		PR2
;
		bclr	PORTB,	#SI				; return SI low
;
		clr		TMR2
		bclr	IFS0,	#T2IF		
		bset	T2CON,	#TON
;
Convert_Loop:
		bclr	PORTB,	#CLK			; clock low
		btg		PORTE,	#WaitToggle		; Test Output		
;						
Convert_Wait1:
		btss	IFS0,	#T2IF
		goto	Convert_Wait1
;
		bclr	IFS0,	#T2IF
		btss	FLAG,	#Sample
		goto	Convert_Expose
;
Convert_Sample:
		bset	ADCON1,	#SAMP			; start a double sample
;
Convert_Sample_Loop:
		btss	ADCON1,	#DONE			; skip next if done
		goto	Convert_Sample_Loop	
;
		mov		ADCBUF0,W0
		swap	W0
		mov.b	W0,	[W2++]
		mov		ADCBUF1,W0
		swap	W0
		mov.b	W0,	[W3++]					
;
Convert_Expose:
		inc		CycleCount
		bset	PORTB,	#CLK			; clock high
;						
Convert_Wait2:
		btss	IFS0,	#T2IF
		goto	Convert_Wait2
;
		bclr	IFS0,	#T2IF
		mov		CycleCount,	W0
		mov		#384,	W1
		cpsgt	W0,		W1				; skip next if W0>W1 (cycle 384)
		goto	Convert_Loop
;
Convert_Done:
		bclr	PORTB,	#CLK			; clock low	
;
		bclr	T2CON,	#TON											
		return		
;                                                                             
;***************************************;***************************************
;
; 	Send Sign On Message
;                                                                             
;***************************************;***************************************
;
SignOn:
		mov 	#tblpage(SignOnI), W0
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(SignOnI), W2
		call	Send2Uart
;		
		mov 	#tblpage(SignOnF), W0
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(SignOnF), W2
		call	Send2Uart
;				
		mov 	#tblpage(SignOnT), W0
		mov 	W0, 	_TBLPAG
		mov 	#tbloffset(SignOnT), W2
		call	Send2Uart		
;				
		return	
;                                                                             
;***************************************;***************************************
;
; 	Send Sign On Message
;                                                                             
;***************************************;***************************************
;
Send2Uart:
		clr		W0
		tblrdl.b	[W2++],	W0
		cp0		W0
		bra		z,		Send2Uart_Exit
;
		mov		W0,		ICHAR_TX
		call	U1RXInterrupt_Output
;
		goto	Send2Uart
;
Send2Uart_Exit:		
		return	
;                                                                             
;***************************************;***************************************
;
; 	Add To Output Buffer
;                                                                             
;***************************************;***************************************
;
AddToOutputBuffer:
		
		return				        
;                                                                             
;***************************************;***************************************
;
; 	Oscillator Fail Trap
;                                                                             
;***************************************;***************************************
;
__OscillatorFail:
        bclr    INTCON1, #OSCFAIL
        nop
        bra     $-2
        retfie
;                                                                             
;***************************************;***************************************
;
; 	Address Error Trap
;                                                                             
;***************************************;***************************************
;
__AddressError:
        bclr    INTCON1, #ADDRERR
        nop
        bra     $-2
        retfie
;                                                                             
;***************************************;***************************************
;
; 	Stack Error Trap
;                                                                             
;***************************************;***************************************
;
__StackError:
        bclr    INTCON1, #STKERR
        nop
        bra     $-2
        retfie
;                                                                             
;***************************************;***************************************
;
; 	Math Error Trap
;                                                                             
;***************************************;***************************************
;
__MathError:
        bclr  INTCON1, #MATHERR
        nop
        bra     $-2
        retfie
;                                                                             
;***************************************;***************************************
;
;--------End of All Code Sections ---------------------------------------------
;                                                                             
;***************************************;***************************************
;
.end                               ;End of program code in this file

