;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Data Memory
;----------------------------------------------------------------------
;----------------------------------------------------------------------

    cblock 0x000 ; Shared RAM (all banks)  LIMITED TO 0x00..0x7F (128 bytes)

; control flags
    pgm_flags           ; global program flags
    diag_flags          ; diagnostics

; configurable charge/analyze settings struct... (stored in EE)
;   must align with SIZEOF_BATCFG value
    batcfg_C:2          ; C (battery rating (e.g. 2200[mAh]))
    batcfg_vFull:2      ; custom Vfull,  in Vx100 units (4.20V typ.) [note that ADC units == Vx100 units * 2]
    batcfg_vEmpty:2     ; custom Vempty, in Vx100 units (3.00V typ.)
    batcfg_limit        ; discharge limit (0..19 = 0.1C to 2.0C)

; LCD display & settings
    LCD_CONST_SSPCON1   ; constants
    LCD_CONST_SSPSTAT   ;   (for quick loading)
    ; EE struct, must align with SIZEOF_DISPCFG value...
    LCD_backlight       ; backlight setting, 0..255 (0=really dim but still on, 255=full on)
    LCD_contrast        ; contrast setting,  0..255 (=> 0..63)

; digital pots (digpot_set routine)
    digpot_save         ; value to write
    digpot_byte         ; shift register
    digpot_count        ; bit counter

; TPA state machine
    tpa_raw:2           ; raw ADC from TPA
    tpa_t0              ; current TPA key code readout
    tpa_t1              ; previous TPA key code readout
    tpa_bounce          ; TPA key code debouncer countdown
    tpa_hold            ; TPA secondary key (holddown) timer
    tpa_key_repeat      ; autorepeat timer
    tpa_key             ; pass key to foregnd (when nonzero)

; type conversion variables
    cnv_value:2         ; input value
    cnv_result          ; result (ASCII buffer) address [IN BANK 3]
    cnv_new:2           ;   temp copy of cnv_value
    cnv_decade:2
    cnv_flags

; page display state
    page_current        ; page == mode [or temp]
    page_base           ;   desired base page
    page_temp           ;   desired temp page
    page_persist        ;     (if persistence > 0)
    page_dirty:((NBR_DYNAMIC_PAGES-1)/8)+1 ; page(s) with pending updates

; timers
    foregnd_20ms_ticks  ; 20 ms ticks to be handled by foregnd
    TIMER_2p5MS         ; 2.5 ms ->  20 ms
    TIMER_20MS          ;  20 ms => 100 ms
    TIMER_100MS         ; 100 ms =>   1 sec
    TIMER_autoreplot    ; page autoreplot (1/2 sec)

; temporary variables
    ; foregnd
    fgnd_temp:6         ; main program
    ee_count            ; ee_readblock
    ee_chksum           ; ee_readblock
    ; interrupt
    int_temp:6          ; interrupt (high only)

; battery operation_request block
    bat_request_data:3  ; [0]=bat_flags, [1,2]=target voltage

; run-time battery data
    ; OWNED BY INTERRUPT [read-only to foregnd]
    bat_flags           ; BAT_...
    bat_analyze         ; ANALYZE_...
    bat_debounce        ; battery insert debouncer (2.5 msec ticks)
    bat_timer           ; dwell timer (1/10 sec ticks = 25.5 secs max)
    bplus_filt:2        ; in ADC units (0.005V / AD)
    bplus_filt_err:2    ; in ADC units (0.005V / AD) x256
    iChg_filt           ; in POT units (0..255)
    iLoad_filt          ; in POT units (0..255)
    bat_vTarget:2       ; target voltage (charging/discharging, Vx100, NOT VALID UNTIL CHARGE/DISCHARGE IS STARTED)
    ; OWNED BY FOREGND
    iChg_limit          ; in POT units, derived from C setting value
    iLoad_limit         ; in POT units, derived from discharge limit setting value
    bat_timeout:2       ; operation time limiter (1 min. ticks)
    bat_1min_timer      ;   1 minute downcounter (1 sec. ticks)
    bat_F_filt          ; F

; heatsink temperature
    heatsink_F_filt     ; F

; current filtering PID
    current_filt        ; in POT units
    current_filt_err:2  ; in POT units x256

; statistical results logging
    log_index           ; offset in log_*_aray buffers
    log_period          ; seconds per sample
    log_timer           ; sample time downcounter
    log_time:3          ; H:M:S (total time, including preconditioning)
    run_time:3          ; charge/dischare time only
    run_accumulator:4   ; charge/discharge total power (mA)
    run_vStart:2        ; ADC units
    run_vEnd:2          ; ADC units
    run_type            ; 0=charge, nonzero=discharge

; interrupted context
    int_save_W
    int_save_STATUS
   ;int_save_BSR

    endc

; pgm_flags
_FLAG_LOADED        EQU B0  ; battery loaded (follows bat_flags)
_FLAG_CLS           EQU B1  ; clear screen is pending
_FLAG_STATICDISPLAY EQU B2  ;   persistent copy of FLAG_CLS for use in page script
_FLAG_ADC           EQU B3  ; ADC read in progress (fault detection)
_FLAG_ERROR         EQU B4  ; dislay error has occurred (for nested error detection)
_FLAG_SHUTDOWN      EQU B5  ; thermal shutdown has occurred

; diag_flags
_DIAG_BLINK         EQU B0  ; blink diagnostic LED for 20ms
_DIAG_WATCHDOG      EQU B7  ; watchdog interlock

; bat_flags
_BAT_LOADED         EQU _FLAG_LOADED ; [B0] battery present (insert debounced, remove instant)
_BAT_CHARGE_ACTIVE  EQU B1
_BAT_CHARGE_CC      EQU B2  ; diag only (all charging is normally CV)
_BAT_LOAD_ACTIVE    EQU B3
_BAT_LOAD_CC        EQU B4
_BAT_DIAGNOSTIC     EQU B5  ; overrides normal limits & shutoffs

; bat_analyze
ANALYZE_OFF          EQU 0  ; stopped (finished)
ANALYZE_PRECONDITION EQU 1  ; precondition (to full from unknown state)
ANALYZE_RUN          EQU 2  ; discharging (to empty), measuring energy

; LCD settings
NBR_SETTINGS        EQU 5   ; number of bytes stored in EE
PERM_SIGNATURE      EQU 0x9876

; TPA key codes
KEY_NONE            EQU 0x00
;KEY_ERROR           EQU 0xF0

KEY_SELECT          EQU 0x15
KEY_HOLD            EQU 0x16
;KEY_RELEASE         EQU 0x12

#if 0
KEY_UP              EQU 0x21
KEY_RIGHT           EQU 0x22
KEY_DOWN            EQU 0x23
KEY_LEFT            EQU 0x24

KEY_REPEAT_UP       EQU 0x41
KEY_REPEAT_RIGHT    EQU 0x42
KEY_REPEAT_DOWN     EQU 0x43
KEY_REPEAT_LEFT     EQU 0x44
#endif

KEY_UP              EQU 0x23
KEY_RIGHT           EQU 0x21
KEY_DOWN            EQU 0x24
KEY_LEFT            EQU 0x22

KEY_REPEAT_UP       EQU 0x43
KEY_REPEAT_RIGHT    EQU 0x41
KEY_REPEAT_DOWN     EQU 0x44
KEY_REPEAT_LEFT     EQU 0x42

; enabled keys (page properties)
#define KEYBIT_SEL  0
#define KEYBIT_HOLD 1
#define KEYBIT_LR   2
#define KEYBIT_UD   3
#define KEYBIT_REP  4

; timer rollovers
_TIMER_2p5MS    EQU 8       ; 2.5 ms=> 20 ms
_TIMER_20MS     EQU 5       ; 20 ms => .1 sec
_TIMER_100MS    EQU 10      ; .1 s  =>  1 sec

; macro to set main display page
SET_BASE_PAGE macro     zPageNum
    MOVLF   zPageNum, page_base
    endm

; macro to set page_temp/page_persist
; example:  SET_POPUP_PAGE POPUP_HD_PAGENUM
SET_POPUP_PAGE macro    zPageNum
    MOVLF   zPageNum,      page_temp
    MOVLF   POPUP_PERSIST, page_persist
    endm

; message display macro
SET_POPUP_MSG macro zMessageCode
    MOVLFF  zMessageCode, context_popup_msg
    SET_POPUP_PAGE POPUP_PAGENUM
    endm

; macro to set page_dirty
;   use only with dynamic pages!
;   example:  SET_DIRTY_PAGE  ABOUT_PAGENUM
SET_DIRTY_PAGE macro zPageCode
    bsf page_dirty+((zPageCode >> 3) & 0x1F), (zPageCode & 0x07)
    endm

; error display macros
SET_ERROR_POPUP macro zErrorCode
    MOVLFF  zErrorCode, context_error
    SET_POPUP_PAGE      ERROR_PAGENUM
    endm

SET_ERROR_FREEZE macro zErrorCode
    MOVLFF  zErrorCode, context_error
    SET_BASE_PAGE       ERROR_PAGENUM
    endm

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

BANK_ADC EQU 1

    cblock (BANK_ADC << 8)

    ; OWNED BY INTERRUPT [read-only to foregnd]
    adc_data_aray:10        ; 5 x words, MUST BE LOCATED AT START OF MEMORY BLOCK

    endc

; data storage indexes (NOT ADC CHANNELS)
#define ADC_DATAINDEX_B1    0   ; ch 0
#define ADC_DATAINDEX_NTC2  1   ;    1 (heatsink)
#define ADC_DATAINDEX_NTC1  2   ;    2 (battery)
#define ADC_DATAINDEX_VREF  3   ;    3 (Vref)
#define ADC_DATAINDEX_TPA   4   ;    4

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; these values must align with generated tables KOMPRESS_TABLE & SKIP_TABLE
LOG_DISPLAY_WIDTH   EQU 100 ; pixels (also nbr of sample points)
LOG_DISPLAY_HEIGHT  EQU 100 ; pixels
LOG_Y_ZERO          EQU  20 ; graph Y origin
LOG_X_ZERO          EQU  20 ; graph X origin

BANK_LOG_V EQU 2

    cblock (BANK_LOG_V << 8)

    ; must be aligned at beginning of bank
    log_V_aray:LOG_DISPLAY_WIDTH*2 ; V/2 x 100 units (0..5.12V)

    endc

BANK_LOG_I EQU 3

    cblock (BANK_LOG_I << 8)

    ; must be aligned at beginning of bank
    log_I_aray:LOG_DISPLAY_WIDTH*2 ; POT units (0..255 ~ 0.050..1.01A)

    endc

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

BANK_DISPLAY EQU 4  ; display communication variables

    cblock (BANK_DISPLAY << 8)

; context_variables are persistent screen navigation variables

    context_mainmenu    ; main menu choice [index]
    context_batmenu     ; battery menu
    context_chargemenu  ; charge menu
    context_testtype    ; analysis menu

    context_settings    ; settings menu

    context_diagmenu    ; diagnostics menu

    context_error       ; error message index

    context_popup_msg   ; popup message index

; display subsystem variables

    ; display orientation
    DISPxOffset     ; 2 (normal) or 0 (inverted)
    DISPyOffset     ; 0 (normal) or 2 (inverted)
    DISPxaset       ; CASET|PASET
    DISPyaset       ; CASET|PASET
    ; display state variables
    DISPx           ; x position
    DISPy           ; y position
    DISPbgnd:2      ; background color
    DISPfgnd:2      ; foreground color
    DISPmode        ; 0=transparent, 1=overstrike
    ; display work variables
    DISPsave:2      ; stash for table pointer (putchar routine)
    DISPstak:2      ; another table pointer copy (for data stacking)
    DISPtemp        ; temp
    DISPchars       ; character counter
    DISPpixels      ; pixel counter
    DISPcols        ; column counter
    DISProws        ; row counter
    DISPscan        ; pixel shift register
    ; current font setting
    FONT_Hpixels    ; column height = pixels
    FONT_Sizes:2    ; font size table (array of bytes)
    FONT_Bitmaps:2  ; font bitmaps (index array followed by bitmap data)
    ; clear screen
    CLS_sr:4        ; 27 bit shift register
    CLS_sra:27

    ; color palette
    ;   splash screen
    PAL_splash_bgnd:2
    PAL_splash_text:2
    ;   system screens
    PAL_popup_text:2
    ;   application screens
    PAL_main_bgnd:2
    PAL_main_text:2
    PAL_title_text:2
    PAL_highlight:2
    PAL_copyright:2
    ; settings screens
    PAL_settings_text:2
    ; graph lines
    PAL_graph_V:2
    PAL_graph_I:2

; shared display variables (only the active page uses a subset of these at a time)

    display_variables ; arbitrary size (see union definitions below)

    endc

; home page
#define display_V      (display_variables+0)  ; #.## format
#define display_I      (display_variables+3)  ; #### format
; results page
#define display_HH     (display_variables+7)  ; 00 format
#define display_MM     (display_variables+9)  ; 00 format
#define display_SS     (display_variables+11) ; 00 format
#define display_mAh    (display_variables+13) ; #### format
#define display_V0     (display_variables+17) ; start V, #.## format
#define display_V1     (display_variables+20) ; end V, #.## format

; interrupted page
#define display_Index    (display_variables+0) ; menu choice index
#define interrupted_page (display_variables+1)

; settings pages
#define display_Original (display_variables+0) ; original setting value (byte/word)
#define display_setting  (display_variables+2) ; XXX % [backlight, contrast] or #.## [voltages] or #### [mA]

; diags pages
#define display_diag_setting (display_variables+0)  ; test choice index, == diag_index - 1 [0..7]
#define display_diag_V       (display_variables+1)  ; #.## format
#define display_diag_chg     (display_variables+4)  ; #### format
#define display_diag_load    (display_variables+8)  ; #### format
#define display_diag_target  (display_variables+12)

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

BANK_MATH EQU 5 ; math library

    cblock (BANK_MATH << 8)

    AARGB0  ; MSB
    AARGB1
    AARGB2
    AARGB3

    BARGB0  ; MSB
    BARGB1
    BARGB2

    REMB0   ; MSB
    REMB1
    REMB2

    MATH_TEMP

    math_save_BSR

    endc

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    cblock 0xF00    ; Bootieloader

    ; all RAM in this bank is reserved

    endc
