;**********************************************************************
;    C O N V E R S I O N     L I B R A R Y     F U N C T I O N S
;
;    Foreground Only
;**********************************************************************

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Byte (0..255) to Percent (0..100)
;
;   y = x * 100/255
;   y*256 = x * 100  (with 0.4% error)
;
cnv_percent

    ; multiply by 100
    movlw   100
    mulwf   cnv_value
    ; divide by 256 (take high byte)
    movff   PRODH, cnv_value
    ; round
    btfsc   PRODL, B7
    incf    cnv_value, f
    ; convert binary to ASCII
    bra     cnv_3

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Binary (8 bit) to ASCII (3 digit integer)
;
cnv_3

    ; set result to "__0"
    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L
    movlw   CHAR_NOBREAKSPACE
    movwf   POSTINC0
    movwf   POSTINC0
    movlw   '0'
    movwf   POSTINC0

cnv_3_common
    ; process all digits
    clrf    cnv_flags
    movff   cnv_result,   FSR0L ; digit 0 = 100's
    MOVLF   100, cnv_decade
    rcall   cnv_1
    movf    POSTINC0, w         ; digit 1 = 10's
    MOVLF    10, cnv_decade
    rcall   cnv_1
    movf    POSTINC0, w         ; digit 2 = 1's
    MOVLF     1, cnv_decade
    bra     cnv_1


DO_CNV_1 macro  ; implemented as macro to reduce stack loading
    local cnv_1_loop, cnv_1_end
    btfss   cnv_flags, B0       ; leading nonzero ?
    bra     cnv_1_loop
    movlw   CHAR_NOBREAKSPACE   ;   yes - convert blanks to zeroes
    cpfseq  INDF0
    bra     cnv_1_loop
    MOVLF   '0', INDF0

cnv_1_loop
    movf    cnv_decade, w       ; test for value < decade
    subwf   cnv_value, f        ;   carry (clear) if value < decade
    bnc     cnv_1_end
    incf    INDF0, f            ; digit = digit + 1
    movlw   CHAR_NOBREAKSPACE+1 ;   test for blank promotion
    cpfseq  INDF0
    bra     cnv_1_loop
    MOVLF   '1', INDF0
    bsf     cnv_flags, B0
    bra     cnv_1_loop

cnv_1_end
    addwf   cnv_value, f        ; restore from underflow
    endm

cnv_2b  ; _# format

    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L
    ; set result to "_0"
    MOVLF   CHAR_NOBREAKSPACE, INDF0
    bra     cnv_2_common

cnv_2   ; ## format

    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L

    ; set result to "00"
    MOVLF   '0', INDF0
cnv_2_common
    movlw   '0'
    movwf   PREINC0

    movlw   100                 ; limit to 00..99
    subwf   cnv_value, w
    movlw   99
    btfsc   STATUS, C
    movwf   cnv_value

    ; process all digits
    clrf    cnv_flags
    movff   cnv_result,   FSR0L ; digit 0 = 10's
    MOVLF    10, cnv_decade
    DO_CNV_1
    movf    POSTINC0, w         ; digit 2 = 1's
    MOVLF     1, cnv_decade
cnv_1 ; DO NOT CALL EXTERNALLY; FOR CNV LOCAL USE ONLY
    DO_CNV_1
    return

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; 16 bit unsigned to ASCII (zero padded)
;

cnv16_3D
    ; set result to "000"
    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L
    movlw   '0'
    movwf   POSTINC0
    movwf   POSTINC0
    movwf   POSTINC0

    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L

    ; process all digits
    setf    cnv_flags
cnv_3D_common
    MOVLF16 100, cnv_decade
    rcall   cnv_1d
    movf    POSTINC0, w
    MOVLF16 10, cnv_decade
    rcall   cnv_1d
    movf    POSTINC0, w
    MOVLF16 1, cnv_decade

cnv_1d
    btfss   cnv_flags, B0       ; leading nonzero ?
    bra     cnv_1d_loop
    movlw   CHAR_NOBREAKSPACE   ;   yes - convert blanks to zeroes
    cpfseq  INDF0
    bra     cnv_1d_loop
    MOVLF   '0', INDF0
cnv_1d_loop
    MOV16   cnv_value,  cnv_new ; make a copy to work with
    SUB16   cnv_decade, cnv_new ; cnv_new = cnv_value - cnv_decade
    btfss   STATUS, C           ;   check for underflow
    return
    ; store value = value - decade
    MOV16   cnv_new, cnv_value
    ; bump ASCII digit
    incf    INDF0, f            ; digit = digit + 1
    movlw   CHAR_NOBREAKSPACE+1 ;   test for blank promotion
    cpfseq  INDF0
    bra     cnv_1d_loop
    MOVLF   '1', INDF0
    bsf     cnv_flags, B0
    bra     cnv_1d_loop

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; 16 bit unsigned to ASCII (#### number)
;

cnv_16_4
    ; set result to "___0"
    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L
    movlw   CHAR_NOBREAKSPACE
    movwf   POSTINC0
    movwf   POSTINC0
    movwf   POSTINC0
    movlw   '0'
    movwf   POSTINC0

    MOVLF   BANK_DISPLAY, FSR0H
    movff   cnv_result,   FSR0L

    ; process all digits
    clrf    cnv_flags
    MOVLF16 1000, cnv_decade
    rcall   cnv_1d
    movf    POSTINC0, w
    bra     cnv_3D_common
