;**********************************************************************
; This function can operate in foregnd or interrupt (uses secondary SPI channel)
; BUT not both at the same time.
;**********************************************************************

;----------------------------------------------------------------------
;----------------------------------------------------------------------
; SPI Digital POTs (MCP41xxx type)
;
; SPI1 is dedicated to the LCD
; SPI2 is available on a chip like PIC18F45K22, but on the '4525 it's bit-banged
;----------------------------------------------------------------------
;----------------------------------------------------------------------

digpot_byte_xmit
    MOVLF   8, digpot_count     ; bit counter
digpot_byte_xmit_loop
    btfss   digpot_byte, B7     ; shift out, MSB first
    bcf     DIGPOT_SDO
    btfsc   digpot_byte, B7
    bsf     DIGPOT_SDO
    bcf     DIGPOT_SCK
    rlncf   digpot_byte, f      ; max 10 MHz clock cycle rate for '41100 @ 5V
    bsf     DIGPOT_SCK          ; data is latched on the rising edge of clock
    decfsz  digpot_count, f
    bra     digpot_byte_xmit_loop
    return

CREATE_DIGPOT_FCN macro zFcnName, zCS_PORT, zCS_BIT

zFcnName

    movwf   digpot_save ; save value to write 

    bcf     zCS_PORT, zCS_BIT

    MOVLF   b'00010001', digpot_byte ; command: Write Data, Potentiometer 0
    rcall   digpot_byte_xmit

    movff   digpot_save, digpot_byte ; potentiometer value
    rcall   digpot_byte_xmit

    bsf     zCS_PORT, zCS_BIT
    return

  endm

; "digipot_set_..." use example:
;
;   movlw   0
;   call    digpot_set_power_1
;   call    digpot_set_load_1

    CREATE_DIGPOT_FCN digpot_set_power_1, DIGPOT_P1_PORT, DIGPOT_P1_BIT
    CREATE_DIGPOT_FCN digpot_set_load_1,  DIGPOT_L1_PORT, DIGPOT_L1_BIT
