;**********************************************************************
;
;    M A I N    P R O G R A M  ---  F O R E G R O U N D
;
;    Allowed to use primary SPI channel only
;    Allowed to use FSR0,FSR1 only
;    Allowed to use TBLPTR
;    Allowed to use Multiply
;
;**********************************************************************

; uses fgnd_temp[0,1]
NTC_INPUT_FILTER macro zADC_index, zResult
    ; look up (F) for NTC ADC value
    GET_INT_WORD adc_data_aray+zADC_index*2, fgnd_temp ; 10 bit
    RR16    fgnd_temp, 2    ; only need 8 bitz
    LOADTBLPTR NTC_TABLE
    movf    fgnd_temp, w
    CALC_TABLE_ADDRESS
    tblrd*  ; read FLASH to TABLAT
    ; feed temperature through filter
    movf    zResult, w
    cpfsgt  TABLAT
    decf    zResult, f
    cpfslt  TABLAT
    incf    zResult, f
    endm

;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------

foreground_tick   ; user program run

;-- watchdog interlock should be set by timebase interrupt every 1/50 sec

    btfss   diag_flags, _DIAG_WATCHDOG
    bra     no_watchdog
    bcf     diag_flags, _DIAG_WATCHDOG
    clrwdt
no_watchdog

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Non-critical timing handler
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

POLL_BAT_TIMERS_100MS macro zTimer ; zTimer in banked RAM
  local xit
    movff   zTimer, WREG        ; get bat_timer
    xorlw   0xFF                ;   0xFF == triggered (stopped)
    bz      xit
    xorlw   0xFF                ;   0x00 == not started (stopped)
    bz      xit
    decf    WREG,w
    btfsc   STATUS, Z           ; trigger when 0 is reached
    setf    WREG
    movff   WREG, zTimer
xit
  endm

;-- loop until all 20 ms ticks have been handled

foregnd_20ms_loop

    movf    foregnd_20ms_ticks, f
    bnz     fgnd_20msec_tick
    goto    foregnd_20ms_end

;-- 20 ms tick

fgnd_20msec_tick
    decf    foregnd_20ms_ticks, f

    ; pop-up page persistence
    tstfsz  page_persist        ; is temp mode active ?
    decf    page_persist, f     ;  yes - count down to zero

    ; timer chain
    decfsz  TIMER_20MS, f
    bra     foregnd_20ms_loop
    MOVLF   _TIMER_20MS, TIMER_20MS

;-- 100 ms tick

    ; autoreplot timer (free running 1/2 sec)
    decfsz  TIMER_autoreplot, f
    bra     fgnd_not_autoreplot
    MOVLF   AUTOREPLOT_TICKS, TIMER_autoreplot
    ;    all these pages replot at 1/2 sec rate...
    SET_DIRTY_PAGE HOME_PAGENUM
    SET_DIRTY_PAGE DIAGRUN_PAGENUM
    SET_DIRTY_PAGE RESULTS_RUN_PAGENUM
fgnd_not_autoreplot

    ; battery monitor timers
    movf    bat_timer, w        ; get bat_timer
    xorlw   0xFF                ;   0xFF == triggered (stopped)
    bz      fgnd_battimer_done
    xorlw   0xFF                ;   0x00 == not started (stopped)
    bz      fgnd_battimer_done
    decf    WREG,w
    btfsc   STATUS, Z           ; trigger when 0 is reached
    setf    WREG
    movff   WREG, bat_timer
fgnd_battimer_done

    ; timer chain
    decfsz  TIMER_100MS, f
    bra     foregnd_20ms_loop
    MOVLF   _TIMER_100MS, TIMER_100MS

;-- 1 sec tick

#if ENABLE_BLINK_ON_1SECTICK == 1
    bsf diag_flags, _DIAG_BLINK
#endif

; temperature sensors

    ; read/convert/filter the temperature sensors
    NTC_INPUT_FILTER ADC_DATAINDEX_NTC1, bat_F_filt
    NTC_INPUT_FILTER ADC_DATAINDEX_NTC2, heatsink_F_filt

    ; special check for already in shutdown state
    btfss   pgm_flags, _FLAG_SHUTDOWN
    bra     fgnd_not_shutdown
    FAN_ON
    btg     LED_RED ; flash at 2 sec rate
    bra     fgnd_1sec_done
fgnd_not_shutdown

    ; temperature failsafes
    movlw   HEATSINK_OVERTEMP
    cpfsgt  heatsink_F_filt
    bra     therm_sink_ok
    ;   heatsink is WAY TOO HOT
    movlw   M_HEATSINK
    bra     therm_shutdown
therm_sink_ok
    movlw   BATTERY_OVERTEMP
    cpfsgt  bat_F_filt
    bra     therm_bat_ok
    ;   battery is overheating
    movlw   M_OVERHEAT
therm_shutdown
    ;   the only exit from thermal shutdown is RESET!
    movff   WREG, context_popup_msg
    SET_BASE_PAGE POPUP_PAGENUM
    bsf     pgm_flags, _FLAG_SHUTDOWN
    call    operation_terminate
    LED_OFF LED_GRN
    LED_OFF LED_RED
    LED_ON  LED_BLU
therm_bat_ok

; fan control

    ; fan is always on during high charge/load conditions ('cause the sense resistors get hot)
    ;   calc total current (in POT units)
    clrf    WREG
    bcf     STATUS, C
    btfsc   bat_flags, _BAT_CHARGE_ACTIVE
    addwf   iChg_filt, w
    btfsc   bat_flags, _BAT_LOAD_ACTIVE
    addwf   iLoad_filt, w
    btfsc   STATUS, C
    setf    WREG
    movwf   fgnd_temp
    ;   check against limit
    movlw   FAN_ON_POT
    cpfslt  fgnd_temp
    bra     fgnd_fan_on

    ; fan is on if heatsink is hot
    movlw   FAN_ON_TEMP
    cpfslt  heatsink_F_filt
    bra     fgnd_fan_on

    ; fan stays on until current is low AND heatsink is cool
    ;   charge/load current
    movlw   FAN_OFF_POT
    cpfslt  fgnd_temp
    bra     fgnd_fan_done
    ;   heatsink temp
    movlw   FAN_OFF_TEMP
    cpfslt  heatsink_F_filt
    bra     fgnd_fan_done

    FAN_OFF
    bra     fgnd_fan_done

fgnd_fan_on
    FAN_ON

fgnd_fan_done

; data logging

    ; only log data when charging/discharging
    btfsc   bat_flags, _BAT_DIAGNOSTIC ; no logging in diagnostic mode
    bra     fgnd_log_done
    movlw   MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_LOAD_ACTIVE)
    andwf   bat_flags, w
    bz      fgnd_log_done

    ; bump total operation timer by 1 second
    lfsr    0, log_time+2
    call    bump_time
    ; update display (time has changed)
    SET_DIRTY_PAGE RESULTS_GRAPH_PAGENUM

    ; no charge/discharge logging during analysis preconditioning phase
    movlw   ANALYZE_PRECONDITION          
    xorwf   bat_analyze, w
    bz      fgnd_log_done

    ; bump specific operation timer
    lfsr    0, run_time+2
    call    bump_time

    ; store voltage/current values to log
    decfsz  log_timer, f
    bra     fgnd_log_VI_done
    ;   check for log buffer full
    movlw   LOG_DISPLAY_WIDTH*2-1
    cpfsgt  log_index
    bra     fgnd_log_go
    ;      compress log buffers to 1/2 size
    call    do_log_compress
    ;      adjust log pointer
    MOVLF   LOG_DISPLAY_WIDTH, log_index
    ;      increase the sampling period by 2X
    rlncf   log_period, f
    btfsc   log_period, B0  ; max period is 255 seconds/sample
    setf    log_period      ;   rail at max period
    ;      replot (CLS) the results page [if active]
    movlw   RESULTS_GRAPH_PAGENUM
    cpfseq  page_current
    bra     fgnd_log_go
    clrf    page_current    ; force screen clear
fgnd_log_go
    ;   store this sample into the log buffer
    movff   log_index, FSR0L
    incf    log_index, f
    ;      current
    MOVLF   high(log_I_aray), FSR0H
    movff   current_filt, INDF0
    ;      voltage
    rcall   get_bplus_to_cnv_value ;MOV16 bplus_filt, cnv_value [ADC units]
    RR16    cnv_value, 2    ; convert ADC units to V/2 x100 (fits in 8 bits)
    MOVLF   high(log_V_aray), FSR0H
    movff   cnv_value, INDF0
    ;   start new sample period
    movff   log_period, log_timer
fgnd_log_VI_done

    ; statistical power tracking
    btfss   bat_flags, _BAT_LOAD_ACTIVE ; only during discharge operation
    bra     fgnd_log_pwr_notload
    ;   totalize mA
    movff   iLoad_filt,             fgnd_temp+0
    MOVLF   MASK(_BAT_LOAD_ACTIVE), fgnd_temp+1
    rcall   calc_current_from_pot ; returns fgnd_temp[4,5] = current [mA], 0..~1K
    ADD16to32 fgnd_temp+4, run_accumulator
fgnd_log_pwr_notload

    btfss   bat_flags, _BAT_CHARGE_ACTIVE ; only during charge operation
    bra     fgnd_log_pwr_notchg
    ;   totalize mA
    movff   iChg_filt,                fgnd_temp+0
    MOVLF   MASK(_BAT_CHARGE_ACTIVE), fgnd_temp+1
    rcall   calc_current_from_pot ; returns fgnd_temp[4,5] = current [mA], 0..~1K
    ADD16to32 fgnd_temp+4, run_accumulator

fgnd_log_pwr_notchg

fgnd_log_done

fgnd_1sec_done

;-- operation time limiter

    ; only active while an operation is in progress
    movf    bat_flags,w
    andlw   MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_LOAD_ACTIVE)
    bz      fgnd_timelimit_done
    ;   AND not diagnostic mode
    btfsc   bat_flags, _BAT_DIAGNOSTIC
    bra     fgnd_timelimit_done
    ; run the timer
    decfsz  bat_1min_timer, f
    bra     fgnd_timelimit_done
    MOVLF   60, bat_1min_timer
    ;   16 bit decrement
    movf    bat_timeout+0, w    ; low byte about to underflow?
    btfsc   STATUS, Z
    decf    bat_timeout+1, f    ; high byte
    decf    bat_timeout+0, f    ; low byte
    ;   test for timeout
    movf    bat_timeout+0, w
    iorwf   bat_timeout+1, w
    bnz     fgnd_timelimit_done
    ;   timed out!
    clrf    bat_analyze ; end all operations
    call    operation_terminate
    LED_OFF LED_GRN
    LED_OFF LED_RED
    LED_ON  LED_BLU
    SET_ERROR_FREEZE E_TIMEOUT

fgnd_timelimit_done

;-- end of foregnd 20ms tick

    bra     foregnd_20ms_loop

foregnd_20ms_end

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Battery State Monitor
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    ; check for battery change-of-state
    movf    bat_flags, w
    xorwf   pgm_flags, w
    andlw   MASK(_FLAG_LOADED)
    bz      fgnd_batmon_end
    ;   battery has been inserted/removed
    btfss   pgm_flags, _FLAG_LOADED
    bra     fgnd_batmon_loaded
    ;   battery has been unloaded
    bcf     pgm_flags, _FLAG_LOADED
    ;   show "Battery Removed" message
    SET_POPUP_MSG M_REMOVED
    ;   IF base page was Battery Menu, goto Main Menu
    movf    page_base, w
    xorlw   BATTERY_MENU_PAGENUM
    movlw   HOME_PAGENUM
    btfsc   STATUS, Z
    movwf   page_base   ; SET_BASE_PAGE
    bra     fgnd_batmon_end

fgnd_batmon_loaded
    bsf     pgm_flags, _FLAG_LOADED
    ; show "BATTERY INSERTED", then the Battery Menu
    SET_BASE_PAGE BATTERY_MENU_PAGENUM
    SET_POPUP_MSG M_INSERTED

fgnd_batmon_end

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Screen Page Manager
;
;   Performs page switching & screen updates
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#define page_desired fgnd_temp+0

foregnd_PageManager

;-- Page state machine

    bcf     pgm_flags, _FLAG_CLS ; assume don't need to clear screen

    ; determine desired page
    movf    page_base, w        ; get base mode
    tstfsz  page_persist        ; temp mode overrides base mode
    movf    page_temp, w        ;    get temp mode
    movwf   page_desired        ; desired page
    ; test for mode change needed
    cpfseq  page_current        ; current page == desired page ?
    bra     prog_lcd_changemode ;   no
    ; check for page update needed
    rcall   check_page_dirty
    bz      prog_lcd_update
    bra     prog_lcd_done       ;   page is not dirty - done

prog_lcd_changemode

    ; change to a new page
    movff   page_desired, page_current
    bsf     pgm_flags, _FLAG_CLS

prog_lcd_update

#if ENABLE_BLINK_ON_PAGEUPDATE == 1
    ; flash diagnostic LED for each page update
    bsf     diag_flags, _DIAG_BLINK
#endif
    ; update all display variables
    CLR     page_dirty, ((NBR_DYNAMIC_PAGES-1)/8)+1
    rcall   do_page_update
    tstfsz  WREG
    bra     foregnd_PageManager ; handle page override in update routine

;-- perform LCD page update

    ; point to page list
    rcall   get_page_tblptr ; set TBLPTR for current_page
    tblrd*+                 ;   skip properties data
    tblrd*+
    tblrd*+
    tblrd*+

    ; let page display code know if this is a new screen or a refresh
    bcf     pgm_flags, _FLAG_STATICDISPLAY
    btfsc   pgm_flags, _FLAG_CLS
    bsf     pgm_flags, _FLAG_STATICDISPLAY

    ; process page data
    call    LCD_display_page

prog_lcd_done

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Key handler
;
; Key event vector table sequence (enabled keys only)
;   SEL
;   HOLD
;   Left
;   Right
;   Up
;   Down
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#define key_code fgnd_temp+0
#define vct_addr fgnd_temp+1

BUMP_VCT_ADDR macro
    movlw   2               ; 2 bytes/entry (table of 'bra' instructions)
    addwf   vct_addr+0, f
    movlw   0
    addwfc  vct_addr+1, f
    endm

foregnd_KeyHandler

    ; get [possible] key
    bcf     INTCON, GIEL    ; mutex with TPA input handler
      movf  tpa_key, w
      clrf  tpa_key
    bsf     INTCON, GIEL
    movwf   key_code        ; save key

    ; process key [if any]
    tstfsz  key_code
    rcall   keyhandler_subr
    bra     keyhandler_done


keyhandler_subr
    ; any key dismisses a popup message
    movf    page_persist,w
    bz      keyhandler_not_popup
    clrf    page_persist
    bra     keyhandler_done
keyhandler_not_popup
    ; ignore keys unless the page is current
    movf    page_current, w
    cpfseq  page_base
    bra     keyhandler_done
    ; convert key code to jump table vector, per current_page properties
    rcall   get_page_tblptr ; set TBLPTR for current_page
    ;   get vector table base address
    tblrd*+
    movff   TABLAT, vct_addr+0
    tblrd*+
    movff   TABLAT, vct_addr+1
    ;   get key selection mask bits
    tblrd*

    btfss   TABLAT, KEYBIT_SEL
    bra     keyhandler_no_SEL
    movlw   KEY_SELECT
    cpfseq  key_code
    bra     keyhandler_not_SEL
keyhandler_go_vector
    movff   vct_addr+1, PCLATH
    movf    vct_addr+0, w
    movwf   PCL

keyhandler_not_SEL
    BUMP_VCT_ADDR           ; key wanted but not matched
keyhandler_no_SEL

    btfss   TABLAT, KEYBIT_HOLD
    bra     keyhandler_no_HOLD
    movlw   KEY_HOLD
    cpfseq  key_code
    bra     keyhandler_not_HOLD
    bra     keyhandler_go_vector
keyhandler_not_HOLD
    BUMP_VCT_ADDR           ; key wanted but not matched
keyhandler_no_HOLD

    btfss   TABLAT, KEYBIT_LR
    bra     keyhandler_no_LR
    movf    key_code, w
    andlw   b'11011111'     ; mask off "normal" key bit
    btfsc   TABLAT, KEYBIT_REP
    andlw   b'10111111'     ; mask off "repeat" key bit
    xorlw   KEY_LEFT & 0x07
    bz      keyhandler_go_vector
    BUMP_VCT_ADDR           ; key wanted but not matched
    movf    key_code, w
    andlw   b'11011111'     ; mask off "normal" key bit
    btfsc   TABLAT, KEYBIT_REP
    andlw   b'10111111'     ; mask off "repeat" key bit
    xorlw   KEY_RIGHT & 0x07
    bz      keyhandler_go_vector
    BUMP_VCT_ADDR           ; key wanted but not matched
keyhandler_no_LR

    btfss   TABLAT, KEYBIT_UD
    bra     keyhandler_no_UD
    movf    key_code, w
    andlw   b'11011111'     ; mask off "normal" key bit
    btfsc   TABLAT, KEYBIT_REP
    andlw   b'10111111'     ; mask off "repeat" key bit
    xorlw   KEY_UP & 0x07
    bz      keyhandler_go_vector
    BUMP_VCT_ADDR           ; key wanted but not matched
    movf    key_code, w
    andlw   b'11011111'     ; mask off "normal" key bit
    btfsc   TABLAT, KEYBIT_REP
    andlw   b'10111111'     ; mask off "repeat" key bit
    xorlw   KEY_DOWN & 0x07
    bz      keyhandler_go_vector
keyhandler_no_UD
    return


keyhandler_done

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; End of foreground
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    goto    BOOT_RETURN ; return to bootloader


;**********************************************************************
;**********************************************************************
;
; Foreground Helpers
;
;**********************************************************************
;**********************************************************************

get_page_tblptr
    ; point to page list
    MOVLF   high(PAGE_LIST_TABLE), TBLPTRH
    MOVLF    low(PAGE_LIST_TABLE), TBLPTRL
    rlncf   page_current, w
    CALC_TABLE_ADDRESS          ; pointing to vector table entry
    tblrd*+
    movf    TABLAT, w
    tblrd*
    movff   TABLAT, TBLPTRH     ; pointing to page list
    movwf           TBLPTRL
    return

;----------------------------------------------------------------------
; Check to see if a page is dirty (needs to be refreshed)
;----------------------------------------------------------------------

check_page_dirty    ; returns Z = 1 if page is dirty

    ; static pages are never dirty
    bcf     STATUS, Z           
    movlw   NBR_DYNAMIC_PAGES
    cpfslt  page_current
    return
    ; get byte for page_current
    lfsr    0, page_dirty       ; multibyte bit array
    rrncf   page_current, w     ;   get byte index for bit number (page_current)
    rrncf   WREG, w
    rrncf   WREG, w
    andlw   b'00011111'
    addwf   FSR0L, f            ;   always in bank 0, so no carry handling is needed
    ; get bit mask for page_current
    MOVLF   high(bitmask_table), TBLPTRH
    MOVLF    low(bitmask_table), TBLPTRL
    movf    page_current, w
    andlw   b'00000111'         ; mod 8
    CALC_TABLE_ADDRESS
    tblrd*
    movf    TABLAT, w
    ; test bit
    movf    INDF0, w            ; dirty page bits
    andwf   TABLAT, w           ; bit to test
    xorwf   TABLAT, w
    return

bitmask_table
    db b'00000001',b'00000010',b'00000100',b'00001000',b'00010000',b'00100000',b'01000000',b'10000000'


;----------------------------------------------------------------------
; Refresh the active page (prepare variables used in the page display)
;
; Returns WREG <> 0 if page has been overridden (changed)
;----------------------------------------------------------------------

do_page_update  ; !!! THIS MUST ALIGN WITH THE "PAGE_LIST_TABLE" !!!

    ; validate page_current as a dynamic page
    movlw   NBR_DYNAMIC_PAGES
    cpfslt  page_current
    retlw   0
    ; jump table - all dynamic pages
    MOVLF   high(page_proc_table), PCLATH
    rlncf   page_current, w         ; get page number, multiply by 2 bytes / table entry
    addlw   low(page_proc_table)
    btfsc   STATUS, C
    incf    PCLATH, f
    movwf   PCL
page_proc_table
    ; must align with PAGE_LIST_TABLE (dynamic paes only)...
    bra     update_home_page        ;  0 home page
    retlw 0                         ;  1 main menu
    retlw 0                         ;  2 settings menu
    bra     update_cont_page        ;  3 contrast settings
    bra     update_bklt_page        ;  4 backlight settings
    bra     update_bat_cap_page     ;  5
    retlw 0                         ;  6 limit setting
    bra     update_bat_Vfull_page   ;  7
    bra     update_bat_Vempty_page  ;  8
    retlw 0                         ;  9 operation interrupt menu
    retlw 0                         ; 10 battery menu
    retlw 0                         ; 11 charge menu
    retlw 0                         ; 12 analyze menu
    retlw 0                         ; 13 diag menu
    bra     update_diag_run_page    ; 14 diag run
    bra     update_results_graph    ; 15 results pages
    bra     update_results_run      ; 16

;- - - - - - - - - - - - - - - - - - - -

update_results_run
    ; was a charge/discharge performed?
    tstfsz  run_time+0
    bra     update_results_go
    tstfsz  run_time+1
    bra     update_results_go
    tstfsz  run_time+2
    bra     update_results_go
    bra     update_results_chg_done
update_results_go
    ; operation time
    MOVLF   low(display_HH), cnv_result
    movff   run_time+0,      cnv_value
    call    cnv_2
    MOVLF   low(display_MM), cnv_result
    movff   run_time+1,      cnv_value
    call    cnv_2
    MOVLF   low(display_SS), cnv_result
    movff   run_time+2,      cnv_value
    call    cnv_2
    ; starting voltage
    MOVLF   low(display_V0), cnv_result
    MOV16   run_vStart,      cnv_value
    RR16    cnv_value,1 ; convert ADC units to V x 100
    call    cnv16_3D
    ; ending voltage
    MOVLF   low(display_V1), cnv_result
    MOV16   run_vEnd,        cnv_value
    RR16    cnv_value,1 ; convert ADC units to V x 100
    call    cnv16_3D
    ; mAh for operation
    ;     = totalized_mAh / 3600
    movff   run_accumulator+3, AARGB0  ; AARG is 31 bits max
    movff   run_accumulator+2, AARGB1
    movff   run_accumulator+1, AARGB2
    movff   run_accumulator+0, AARGB3
    MOVLFF  0,         BARGB0          ; BARG is 23 bits max
    MOVLFF  high(3600),BARGB1
    MOVLFF   low(3600),BARGB2
    call    FXD3123U ; result in AARGB0,1,2,3 and should not exceed 16 bits
    MOVLF16 9999, cnv_value
    lfsr    0, AARGB0
    tstfsz  POSTINC0
    bra     update_results_mAh_rdy ; overflowed 24 bitz [eh?]
    tstfsz  POSTINC0
    bra     update_results_mAh_rdy ; overflowed 16 bitz [eh?]
    movff   POSTINC0, cnv_value+1
    movff   INDF0,    cnv_value+0
update_results_mAh_rdy
    MOVLF   low(display_mAh), cnv_result
    call    cnv_16_4
    ; voltage now
    MOVLF   low(display_V), cnv_result
    rcall   get_bplus_to_cnv_value ;MOV16 bplus_filt, cnv_value
    RR16    cnv_value,1 ; convert ADC units to V x 100
    call    cnv16_3D
    ; current now
    movff   iChg_filt,               fgnd_temp+0
    MOVLF   MASK(_BAT_CHARGE_ACTIVE),fgnd_temp+1
    btfsc   bat_flags, _BAT_CHARGE_ACTIVE
    bra     update_home_I
    movff   iLoad_filt,             fgnd_temp+0
    MOVLF   MASK(_BAT_LOAD_ACTIVE), fgnd_temp+1
    btfsc   bat_flags, _BAT_LOAD_ACTIVE
    bra     update_results_I
    CLR     fgnd_temp+4,2
    bra     update_results_I0
update_results_I
    rcall   calc_current_from_pot
update_results_I0
    MOV16   fgnd_temp+4,    cnv_value
    MOVLF   low(display_I), cnv_result
    call    cnv_16_4
update_results_chg_done
    retlw   0

;- - - - - - - - - - - - - - - - - - - -

update_results_graph
    ; NOTE - results page also calls paint_results below
    MOVLF   low(display_HH), cnv_result
    movff   log_time+0,      cnv_value
    call    cnv_2
    MOVLF   low(display_MM), cnv_result
    movff   log_time+1,      cnv_value
    call    cnv_2
    MOVLF   low(display_SS), cnv_result
    movff   log_time+2,      cnv_value
    call    cnv_2
    retlw   0

;- - - - - - - - - - - - - - - - - - - -
; display logic - for the case where there's at least 1X buffer of data
;
;   IF buffer is 1X, step == 1 (super-trivial case)
;   IF buffer is 2X, step == 2 (almost as trivial)
;   ELSE it's a non-integer step (not so trival)
;
;   BASICALLY, U want to split up the remainder (buf_index - display_width)
;              over the display_width.
;
;   GIVEN:  magic_number = round( 32768 * extra_pixels / display_width )
;   WHERE:  extra_pixels is 0..display_width
;
;   SET skip_accumulator = 16384
;   FOR each pixel displayed, add magic_number to skip_accumulator
;    IF skip_accumulator > 15 bits, THEN skip a pixel AND subtract 32768 from skip_accumulator
;
;   easy, eh?
;
; for the case where there's less than 1X buffer (< 100 secs)
;
;   magic_number = 32768 and use same logic as above
;   EXCEPT that only the amount of data in the buffer is displayed

#define skip_accumulator fgnd_temp+0 ; fgnd_temp[0:1] is skip_accumulator
#define magic_number     fgnd_temp+2 ; fgnd_temp[2:3] is magic_number
#define save_TBLPTR      fgnd_temp+4 ; must preserve TBLPTR (LCD library is using it!)

paint_results_I_graph ; called from LCD library (BSR is set properly for LCD library use)

    lfsr    1, log_I_aray
    bra     paint_results_common

paint_results_V_graph ; called from LCD library (BSR is set properly for LCD library use)

    lfsr    1, log_V_aray

paint_results_common
    movff   TBLPTRL, save_TBLPTR+0
    movff   TBLPTRH, save_TBLPTR+1
    MOVLFB  LOG_X_ZERO, DISPx
    ; look up magic_number for extra_pixels
    LOADTBLPTR SKIP_TABLE
    ;    IF actively recording data, magic number is 32768 (always skip 1 pixel)
    movlw   RESULTS_GRAPH_PAGENUM
    xorwf   page_current,w
    bz      print_results_2X
    ;    calc extra_pixels [log_index - LOG_DISPLAY_WIDTH]
    movlw   -LOG_DISPLAY_WIDTH
    addwf   log_index, w
    ;    if not full width, magic number is 32768
    btfss   STATUS, C
print_results_2X
    movlw   LOG_DISPLAY_WIDTH
    ;    look up magic number
    rlncf   WREG,w  ; 2 bytes/entry in SKIP_TABLE
    CALC_TABLE_ADDRESS
    tblrd*+ ; low byte
    movff   TABLAT, magic_number+0
    tblrd*  ; high byte
    movff   TABLAT, magic_number+1
    ; init skip accumulator for balanced skipping
    MOVLF16 16384, skip_accumulator

paint_results_loop
    ; quit early if the buffer is not full
    movf    log_index, w
    cpfslt  FSR1L
    bra     paint_results_xit
    ; compress Y dimension from 0..255 to graph height
    LOADTBLPTR KOMPRESS_TABLE
    movf    POSTINC1, w
    CALC_TABLE_ADDRESS
    tblrd*
    movf    TABLAT, w
    addlw   LOG_Y_ZERO
    movwf   DISPy, BANKED
    call    LCD_putpixel
    incf    DISPy, f, BANKED        ; 2-pixel thick line
    call    LCD_putpixel
    ; skip sample point if appropriate
    ADD16   magic_number, skip_accumulator
    btfss   skip_accumulator+1, B7  ; test for overflowing 15 bits
    bra     paint_results_noskip
    movf    POSTINC1, w             ; skip this sample
    bcf     skip_accumulator+1, B7  ; subtract 32768 from skip_accumulator [we know that B15 is set]
paint_results_noskip
    ; advance pixel X coordinate
    incf    DISPx, f, BANKED
    movlw   LOG_X_ZERO+LOG_DISPLAY_WIDTH
    cpfsgt  DISPx, BANKED
    bra     paint_results_loop
paint_results_xit
    ; restore TBLPTR for return to LCD library
    movff   save_TBLPTR+1, TBLPTRH
    movff   save_TBLPTR+0, TBLPTRL
    return

;- - - - - - - - - - - - - - - - - - - -

update_home_page

; voltage
    MOVLF   low(display_V), cnv_result
    rcall   get_bplus_to_cnv_value ;MOV16 bplus_filt, cnv_value
    RR16    cnv_value,1 ; convert ADC units to V x 100
    call    cnv16_3D

; current
    movff   iChg_filt,               fgnd_temp+0
    MOVLF   MASK(_BAT_CHARGE_ACTIVE),fgnd_temp+1
    btfsc   bat_flags, _BAT_CHARGE_ACTIVE
    bra     update_home_I
    movff   iLoad_filt,             fgnd_temp+0
    MOVLF   MASK(_BAT_LOAD_ACTIVE), fgnd_temp+1
    btfsc   bat_flags, _BAT_LOAD_ACTIVE
    bra     update_home_I
    CLR     fgnd_temp+4,2
    bra     update_home_I0
update_home_I
    rcall   calc_current_from_pot
update_home_I0
    MOV16   fgnd_temp+4,    cnv_value
    MOVLF   low(display_I), cnv_result
    call    cnv_16_4

    retlw   0

;- - - - - - - - - - - - - - - - - - - -

update_diag_run_page

; Charge current
    movff   iChg_filt,               fgnd_temp+0
    MOVLF   MASK(_BAT_CHARGE_ACTIVE),fgnd_temp+1
    rcall   calc_current_from_pot
    MOV16   fgnd_temp+4,             cnv_value
    MOVLF   low(display_diag_chg),   cnv_result
    call    cnv_16_4

; Load current
    movff   iLoad_filt,             fgnd_temp+0
    MOVLF   MASK(_BAT_LOAD_ACTIVE), fgnd_temp+1
    rcall   calc_current_from_pot
    MOV16   fgnd_temp+4,            cnv_value
    MOVLF   low(display_diag_load), cnv_result
    call    cnv_16_4

; voltages
    MOVLF   low(display_diag_target), cnv_result
    MOV16   bat_vTarget,              cnv_value
    call    cnv16_3D

    MOVLF   low(display_diag_V), cnv_result
    rcall   get_bplus_to_cnv_value ;MOV16 bplus_filt, cnv_value
    RR16    cnv_value,1 ; convert ADC units to V x 100
    call    cnv16_3D

    retlw   0

; I(mA) = 50 + POT*4 - POT/8 + POT/32    [result is within +/- 1 mA of exact]

calc_current_from_pot
    ; Sent: fgnd_temp[0] = current setting (charge/load) in POT units
    ;       fgnd_temp[1] = active MOSFET switch mask
    ; Rets: fgnd_temp[4,5] = current [mA], 0..~1K

    ; show zero if the MOSFET is off
    CLR     fgnd_temp+4, 2
    movf    bat_flags, w    ; bat_flags
    andwf   fgnd_temp+1, w  ;   active switch mask
    bz      ccfp_go

    MOVLF16 50, fgnd_temp+4 ; result accumulator

    clrf    fgnd_temp+1     ; POT setting now 16 bit

    MOV16   fgnd_temp, fgnd_temp+2
    RL16    fgnd_temp+2, 2  ; POT * 4
    ADD16   fgnd_temp+2, fgnd_temp+4

    MOV16   fgnd_temp, fgnd_temp+2
    RR16    fgnd_temp+2, 3  ; POT / 8
    SUB16   fgnd_temp+2, fgnd_temp+4
    RR16    fgnd_temp+2, 2  ; POT / 32
    ADD16   fgnd_temp+2, fgnd_temp+4
ccfp_go
    return

;- - - - - - - - - - - - - - - - - - - -
; Display Settings pages

update_cont_page

    movff   LCD_contrast,         cnv_value

update_setting_common
    MOVLF   low(display_setting), cnv_result
    call    cnv_percent
    retlw 0

update_bklt_page

    movff   LCD_backlight,        cnv_value
    bra     update_setting_common

;- - - - - - - - - - - - - - - - - - - -
; Battery Settings pages

update_bat_cap_page

    MOVLF   low(display_setting), cnv_result
    MOV16   batcfg_C,             cnv_value
    call    cnv_16_4
    retlw   0

update_bat_Vfull_page

    MOVLF   low(display_setting), cnv_result
    MOV16   batcfg_vFull,         cnv_value
    call    cnv16_3D
    retlw   0

update_bat_Vempty_page

    MOVLF   low(display_setting), cnv_result
    MOV16   batcfg_vEmpty,        cnv_value
    call    cnv16_3D
    retlw   0

;----------------------------------------------------------------------
; page display helpers

get_bplus_to_cnv_value
    GET_INT_WORD bplus_filt, cnv_value
    return


;----------------------------------------------------------------------
; Key Handlers
;
; all fgnd_temp variables are available for use by the key handlers
;----------------------------------------------------------------------

;- - - - - - - - - - - - - - - - - - - -
; home (consolidated status display)
;- - - - - - - - - - - - - - - - - - - -

home_activate
    SET_BASE_PAGE HOME_PAGENUM
    return

keyjumptab_home
    bra home_SEL                ; SEL
    bra results_graph_activate  ; L
    bra results_run_activate    ; R
    bra main_activate           ; U
                                ; D
    btfsc   pgm_flags, _FLAG_LOADED ; IF battery_present
    bra     batmenu_activate        ;   THEN goto battery_menu
    SET_POPUP_MSG M_NOBATTERY
    return

home_SEL
    movf    bat_flags,w     ; IF operation_in_progress
    andlw   MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_LOAD_ACTIVE)
    bz      main_activate   ;   THEN goto main_menu
    bra     batint_activate ;   ELSE goto interrupt_menu

;- - - - - - - - - - - - - - - - - - - -
; main menu
;- - - - - - - - - - - - - - - - - - - -

main_activate
    SET_BASE_PAGE MAIN_PAGENUM
    return

keyjumptab_main 
    bra main_SEL
    bra home_activate   ; L
    bra main_SEL        ; R
    bra main_U
main_D
    movff   context_mainmenu, WREG
    btfsc   WREG,B2 ; test for WREG==4 without destroying WREG
    setf    WREG
    addlw   1
main_set
    movff   WREG, context_mainmenu
    SET_DIRTY_PAGE MAIN_PAGENUM
    return

main_U
    movff   context_mainmenu, WREG
    decf    WREG,w
    btfsc   WREG,B7
    movlw   4
    bra     main_set

main_SEL
    movff   context_mainmenu, WREG
    decf    WREG,w
    bz      settings_activate   ; 1
    dcfsnz  WREG,w
    goto    diagnostics_menu_activate ; 2
    decf    WREG,w
    bz      about_activate      ; 3
    decf    WREG,w
    bz      home_activate       ; 4
main_SEL_battery ; ambiguous menu item
    btfsc   pgm_flags, _FLAG_LOADED ; IF battery_present
    bra     batmenu_activate        ;   THEN goto battery_menu
    SET_POPUP_MSG M_NOBATTERY
    return

;- - - - - - - - - - - - - - - - - - - -
; About page
;- - - - - - - - - - - - - - - - - - - -

about_activate
    SET_BASE_PAGE ABOUT_PAGENUM
    return

keyjumptab_about
    bra about_ALL ; SEL
    bra about_ALL ; L
    bra about_ALL ; R
    bra about_ALL ; U
about_ALL         ; D
    bra main_activate

;- - - - - - - - - - - - - - - - - - - -
; error page
;- - - - - - - - - - - - - - - - - - - -

; page activation requirement:  "context_error" must be set
;                               (use SET_ERROR_POPUP/FREEZE macro)
keyjumptab_error
    bra home_activate ; & return

;- - - - - - - - - - - - - - - - - - - -
; Settings menu / pages
;- - - - - - - - - - - - - - - - - - - -

settings_activate
    SET_BASE_PAGE SETTINGS_MENU_PAGENUM
    return

keyjumptab_settings
    bra     settings_SEL
    bra     settings_U
settings_D
    movff   context_settings, WREG
    btfsc   WREG,B1 ; test for WREG==2 without destroying WREG
    setf    WREG
    incf    WREG,w
settings_set
    movff   WREG, context_settings
    SET_DIRTY_PAGE SETTINGS_MENU_PAGENUM
settings_return
    return

settings_U
    movff   context_settings, WREG
    decf    WREG,w
    btfsc   WREG,B7
    movlw   2
    bra     settings_set

settings_SEL
    movff   context_settings, WREG
    movf    WREG,w
    bz      cont_activate
    decf    WREG,w
    bz      bklt_activate
    bra     main_activate

; backlight setting page

bklt_activate
    movff   LCD_backlight, display_Original ; for changed data detection
    SET_BASE_PAGE SETTINGS_BKLT_PAGENUM
    return

keyjumptab_bklt
    bra bklt_SEL
    bra bklt_U
bklt_D
    tstfsz  LCD_backlight       ; rail at 0
    decf    LCD_backlight, f
    bra     bklt_UD_common

bklt_U
    incf    LCD_backlight, w
    btfss   STATUS, Z           ; rail at 0xFF
    incf    LCD_backlight, f
bklt_UD_common
    LCD_SET_BACKLIGHT
    SET_DIRTY_PAGE SETTINGS_BKLT_PAGENUM
    return

bklt_SEL
    rcall   settings_activate

bklt_save_changes ; update EE [only if value was changed]

    ; check for data changed
    movff   display_Original, WREG
    xorwf   LCD_backlight, w
    btfsc   STATUS,Z
    return

store_display_settings_common

    ; store settings to EE
    MOVLF   EEADR_DISPLAY, EEADR
    lfsr    0, LCD_backlight
    movlw   SIZEOF_DISPCFG
    goto    EE_storeblock       ; & return

; contrast setting page

cont_activate
    movff   LCD_contrast, display_Original ; for changed data detection
    SET_BASE_PAGE SETTINGS_CONT_PAGENUM
    return

keyjumptab_cont
    bra cont_SEL
    bra cont_U
cont_D
    tstfsz  LCD_contrast        ; rail at 0
    decf    LCD_contrast, f
    bra     cont_UD_common

cont_U
    incf    LCD_contrast, w
    btfss   STATUS, Z           ; rail at 0xFF
    incf    LCD_contrast, f
cont_UD_common
    call    LCD_set_contrast
    SET_DIRTY_PAGE SETTINGS_CONT_PAGENUM
    return

cont_SEL
    rcall   settings_activate

cont_save_changes ; update EE [only if value was changed]

    ; check for data changed
    movff   display_Original, WREG
    xorwf   LCD_contrast, w
    btfsc   STATUS,Z
    return

    ; store settings to EE
    bra     store_display_settings_common

;- - - - - - - - - - - - - - - - - - - -
; Battery Menu page
;- - - - - - - - - - - - - - - - - - - -

batmenu_activate
    ; activate interrupt page if an operation is in progress now
    movf    bat_flags, w
    andlw   MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_LOAD_ACTIVE)
    bnz     batint_activate
    SET_BASE_PAGE BATTERY_MENU_PAGENUM
    return

keyjumptab_batmenu
    bra batmenu_SEL
    bra home_activate   ; L
    bra batmenu_SEL     ; R
    bra batmenu_U
batmenu_D
    movff   context_batmenu, WREG
    addlw   1
batmenu_set
    andlw   0x03
    movff   WREG, context_batmenu
    SET_DIRTY_PAGE BATTERY_MENU_PAGENUM
    return

batmenu_U
    movff   context_batmenu, WREG
    addlw   3
    bra     batmenu_set

batmenu_SEL
    movff   context_batmenu, WREG
    decf    WREG,w
    bz      batanalyze_activate ; choice #1
    decf    WREG,w
    bz      batmenu_SEL_2    
    decf    WREG,w
    bz      batmenu_SEL_3    
batmenu_SEL_0 ; Charge
    rcall   batcharge_activate  ;SET_BASE_PAGE BATTERY_CHARGE_PAGENUM
    return

batmenu_SEL_2 ; Results
    bra     results_activate

batmenu_SEL_3 ; Exit
    bra     home_activate

;- - - - - - - - - - - - - - - - - - - -
; Interrupted operation menu
;- - - - - - - - - - - - - - - - - - - -

batint_activate
    movf    bat_flags, w
    andlw   MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_LOAD_ACTIVE)
    bz      batmenu_SEL_3 ; far jump to home_activate
    ; activate interrupt page
    movff   page_current, interrupted_page
    MOVLFF  0,display_Index ; always default to 1'st choice ("STOP")
    SET_BASE_PAGE BATTERY_INT_PAGENUM
    return

keyjumptab_batint
    bra batint_SEL
    bra batint_resume   ; L [previous screen]
    bra home_activate   ; R [home]
    bra batint_U
batint_D
    movff   display_Index, WREG
    btfsc   WREG,B1     ; test for WREG == 2
    setf    WREG
    incf    WREG,w
batint_set
    movff   WREG, display_Index
    SET_DIRTY_PAGE BATTERY_INT_PAGENUM
    return

batint_U
    movff   display_Index, WREG
    decf    WREG,w
    btfsc   WREG,B7
    movlw   2
    bra     batint_set

batint_SEL
    movff   display_Index, WREG
    decf    WREG,w
    bz      batmenu_SEL_2       ; far goto link ["Results"]
    decf    WREG,w
    bz      batint_resume
    ; default [item 0] is "STOP"
    clrf    bat_analyze         ; stop analyzer
    call    operation_terminate ; stop current operation
    LED_OFF LED_RED
    LED_OFF LED_GRN
    bra     results_activate    ; show results

batint_resume ; return to previous screen
    movff   interrupted_page, page_base
    return

;- - - - - - - - - - - - - - - - - - - -
; Analyze menu
;- - - - - - - - - - - - - - - - - - - -

batanalyze_activate
    SET_BASE_PAGE BATTERY_ANALYZE_PAGENUM
    return

keyjumptab_batanalyze
    bra batanalyze_SEL
    bra batmenu_activate ; L
    bra batanalyze_SEL   ; R
    bra batanalyze_U
batanalyze_D
    movff   context_testtype, WREG
    btfsc   WREG,B1     ; test for WREG == 2
    setf    WREG
    incf    WREG,w
batanalyze_set
    movff   WREG, context_testtype
    SET_DIRTY_PAGE BATTERY_ANALYZE_PAGENUM
    return

batanalyze_U
    movff   context_testtype, WREG
    decf    WREG,w
    btfsc   WREG,B7
    movlw   2
    bra     batanalyze_set

batanalyze_SEL
    movff   context_testtype, WREG
    xorlw   2   ; "Exit" selected?
    bz      batmenu_activate
; FALL THRU to batcharge_activate...

;- - - - - - - - - - - - - - - - - - - -
; Charge menu
;- - - - - - - - - - - - - - - - - - - -

batcharge_activate
    SET_BASE_PAGE BATTERY_CHARGE_PAGENUM
    return

keyjumptab_batcharge
    bra batcharge_SEL
    bra batcharge_L     ; L
    bra batcharge_SEL   ; R
    bra batcharge_U
batcharge_D
    movff   context_chargemenu, WREG
    btfsc   WREG,B2     ; test for WREG==4 without destroying WREG
    setf    WREG
    incf    WREG,w
batcharge_set
    movff   WREG, context_chargemenu
    SET_DIRTY_PAGE BATTERY_CHARGE_PAGENUM
batcharge_return
    return

batcharge_U
    movff   context_chargemenu, WREG
    decf    WREG,w
    btfsc   WREG,B7
    movlw   4
    bra     batcharge_set

batcharge_L ; previous screen can be battery menu or analysis menu
    movff   context_batmenu, WREG
    tstfsz  WREG
    bra     batanalyze_activate
    bra     batmenu_activate

batcharge_SEL
    movff   context_chargemenu, WREG
    addlw   -3      ; choice 3 == "Custom"
    bz      vFull_activate
    dcfsnz  WREG,w  ; choice 4 == "Exit"
    bra     home_activate
; FALL THRU to capacity_activate...

;- - - - - - - - - - - - - - - - - - - -
; Battery capacity setting page
;- - - - - - - - - - - - - - - - - - - -

capacity_activate
    MOV16   batcfg_C, display_Original ; for changed data detection
    SET_BASE_PAGE BAT_CAPACITY_PAGE
    return

keyjumptab_capacity
    bra capacity_SEL
    bra capacity_L
    bra capacity_R
    bra capacity_U
capacity_D ; adjust value
    MOV16   batcfg_C, fgnd_temp
    SUBL16  MIN_CAPACITY+100, fgnd_temp
    btfss   STATUS, C
    return  ; rail at 1000
    SUBL16  100, batcfg_C
capacity_update
    SET_DIRTY_PAGE BAT_CAPACITY_PAGE
    return

capacity_U ; adjust value
    MOV16   batcfg_C, fgnd_temp
    SUBL16  MAX_CAPACITY-99, fgnd_temp
    btfsc   STATUS, C
    return  ; rail at 3000
    ADDL16  100, batcfg_C
    bra     capacity_update

capacity_L ; previous screen can be charge_type or vEmpty
    rcall   capacity_save_changes
    movff   context_chargemenu, WREG
    xorlw   3   ; choice 3 == "Custom"
    bz      vEmpty_activate
    bra     batcharge_activate

capacity_R ; next screen
capacity_SEL
    rcall   capacity_save_changes
    bra     limit_activate

capacity_save_changes
    lfsr    0, batcfg_C

batcfg_save16 ; common save helper

    movff   display_Original+1, WREG
    xorwf   PREINC0, w    
    bnz     batcfg_save_go
    movf    POSTDEC0, w

batcfg_save8

    movff   display_Original+0, WREG
    xorwf   INDF0, w
    bnz     batcfg_save_go
    return  ; data was not changed

batcfg_save_go

    MOVLF   EEADR_BATTERY, EEADR
    lfsr    0, batcfg_C
    movlw   SIZEOF_BATCFG
    goto    EE_storeblock ; & return

;- - - - - - - - - - - - - - - - - - - -
; Discharge rate limit setting page
;- - - - - - - - - - - - - - - - - - - -

limit_activate
    movff   batcfg_limit, display_Original ; for changed data detection
    SET_BASE_PAGE BAT_LIMIT_PAGE
    return

keyjumptab_limit
    bra limit_SEL
    bra limit_L
    bra limit_R
    bra limit_U
; Limit multiplier = X * 2 [* 256]
;  0 = 0.1C =>  26
;   ...
; 18 = 1.9C => 486
; 19 = 2.0C => 512

limit_D ; adjust value down
    movf    batcfg_limit, w
    bz      limit_done ; rail at low limit
    decf    batcfg_limit, f
limit_update
    SET_DIRTY_PAGE BAT_LIMIT_PAGE
limit_done
    return

limit_U ; adjust value up
    movlw   19
    cpfslt  batcfg_limit
    bra     limit_done ; rail at high limit
    incf    batcfg_limit, f
    bra     limit_update

limit_L ; previous screen
    rcall   limit_save_changes
    bra     capacity_activate

limit_SEL ; next screen [start operation, then Home]
limit_R
    rcall   limit_save_changes
    ; calculate charge/discharge limit values
    rcall   calc_limits
    ; next screen depends on context
    movff   context_batmenu, WREG
    tstfsz  WREG
    bra     start_analyzer
    ;   simple charge/discharge
    SET_POPUP_MSG M_CHARGING ; "Charge Started"
    SET_BASE_PAGE HOME_PAGENUM
    bra     charge_start

start_analyzer
    SET_POPUP_MSG M_ANALYZING ; "Analysis Started"
    SET_BASE_PAGE HOME_PAGENUM
    goto    analyzer_activate

limit_save_changes
    lfsr    0, batcfg_limit
    bra     batcfg_save8

;- - - - - - - - - - - - - - - - - - - -
; Custom Battery setup screens
;- - - - - - - - - - - - - - - - - - - -

; "full" (cut-off) voltage setting

vFull_activate
    MOV16   batcfg_vFull, display_Original ; for changed data detection
    SET_BASE_PAGE BAT_VFULL_PAGE
    return

keyjumptab_Vfull
    bra vFull_SEL
    bra vFull_L
    bra vFull_R
    bra vFull_U
vFull_D
    MOV16   batcfg_vFull, fgnd_temp
    SUBL16  300+1, fgnd_temp
    btfss   STATUS, C
    return  ; rail at 3.0V
    SUBL16  1, batcfg_vFull
    SET_DIRTY_PAGE BAT_VFULL_PAGE
    return

vFull_U
    MOV16   batcfg_vFull, fgnd_temp
    SUBL16  450, fgnd_temp
    btfsc   STATUS, C
    return  ; rail at 4.5V
    ADDL16  1, batcfg_vFull
    SET_DIRTY_PAGE BAT_VFULL_PAGE
    return

vFull_L
    rcall   vFull_save_changes
    bra     batcharge_activate ; previous screen
    return

vFull_save_changes
    lfsr    0, batcfg_vFull
    bra     batcfg_save16

vFull_R
vFull_SEL
    rcall   vFull_save_changes
; FALL THRU to vEmpty_activate...

; "empty" (cut-off) voltage setting

vEmpty_activate
    MOV16   batcfg_vEmpty, display_Original ; for changed data detection
    SET_BASE_PAGE BAT_VEMPTY_PAGE
    return

keyjumptab_Vempty
    bra vEmpty_SEL
    bra vEmpty_L
    bra vEmpty_R
    bra vEmpty_U
vEmpty_D
    MOV16   batcfg_vEmpty, fgnd_temp
    SUBL16  200+1, fgnd_temp
    btfss   STATUS, C
    return  ; rail at 2.0V
    SUBL16  1, batcfg_vEmpty
    SET_DIRTY_PAGE BAT_VEMPTY_PAGE
    return

vEmpty_U
    MOV16   batcfg_vEmpty, fgnd_temp
    SUBL16  400, fgnd_temp
    btfsc   STATUS, C
    return  ; rail at 4.0V
    ADDL16  1, batcfg_vEmpty
    SET_DIRTY_PAGE BAT_VEMPTY_PAGE
    return

vEmpty_L
    rcall   vEmpty_save_changes
    bra     vFull_activate ; previous screen

vEmpty_R
vEmpty_SEL
    rcall   vEmpty_save_changes
    bra     capacity_activate

vEmpty_save_changes
    lfsr    0, batcfg_vEmpty
    bra     batcfg_save16

;- - - - - - - - - - - - - - - - - - - -
; Results screens
;- - - - - - - - - - - - - - - - - - - -

results_activate
    ; IF analyzer_in_precondition_state
    movlw   ANALYZE_PRECONDITION
    cpfseq  bat_analyze
    bra     results_graph_activate
    ;   THEN no results available yet
    SET_POPUP_MSG M_NORESULTS
    return

results_graph_activate
    SET_BASE_PAGE RESULTS_GRAPH_PAGENUM
    return

keyjumptab_results_graph
    bra batint_activate         ; SEL
    bra results_run_activate    ; left
    bra results_run_activate    ; right
    bra batint_activate         ; up
    bra home_activate           ; down

;- - - - - - - - - - - - - - - - - - - -

results_run_activate
    SET_BASE_PAGE RESULTS_RUN_PAGENUM
    return

keyjumptab_results_run
    bra batint_activate         ; SEL
    bra results_graph_activate  ; L
    bra results_graph_activate  ; R
    bra batint_activate         ; up
    bra home_activate           ; down

;- - - - - - - - - - - - - - - - - - - -
; Diagnostic menu
;- - - - - - - - - - - - - - - - - - - -

diagnostics_menu_activate
    SET_BASE_PAGE DIAGMENU_PAGENUM
    return

keyjumptab_diagmenu
    bra diagmenu_SEL
    bra diagmenu_LR
    bra diagmenu_LR
    bra diagmenu_U
diagmenu_D
    MOVLF   10,fgnd_temp+0
    MOVLF   5, fgnd_temp+1
    movff   context_diagmenu, WREG
    incf    WREG,w
    xorwf   fgnd_temp+0, f
    xorwf   fgnd_temp+1, f
    btfsc   STATUS, Z
    movlw   0           ; 4 wraps to 0
    movf    fgnd_temp+0, f
    btfsc   STATUS, Z
    movlw   5           ; 9 wraps to 5
diagmenu_set
    movff   WREG, context_diagmenu
    SET_DIRTY_PAGE DIAGMENU_PAGENUM
    return

diagmenu_U
    MOVLF      4,fgnd_temp+0
    MOVLF   0xFF,fgnd_temp+1
    movff   context_diagmenu, WREG
    decf    WREG,w
    xorwf   fgnd_temp+0, f
    xorwf   fgnd_temp+1, f
    btfsc   STATUS, Z
    movlw   4           ; 0 wraps to 4
    movf    fgnd_temp+0, f
    btfsc   STATUS, Z
    movlw   9           ; 5 wraps to 9
    bra     diagmenu_set

diagmenu_LR
    MOVLF   10, fgnd_temp+0
    movff   context_diagmenu, WREG
    addlw   5
    cpfsgt  fgnd_temp+0
    addlw   -10
    bra     diagmenu_set

diagmenu_SEL
    movff   context_diagmenu, WREG
    movf    WREG,w
    bz      diag_SEL_0
    bra     diagnostics_run_activate ; & return

diag_SEL_0 ; return to home page
    bra     main_activate

;- - - - - - - - - - - - - - - - - - - -
; Diagnostics
;
;   Standard Test Configuration:
;
;       Voltage meter (2V)  at R2 (1A == 1V)
;       Voltage meter (10V) at B+/B-
;       for CV tests, add 100 uF capacitor at B+/B-

diagnostics_parm_table

 ; CC load - connect current meter to test point, verify ~100 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), POT_FULL,  POT_100MA, 0

 ; CC load - connect current meter to test point, verify ~500 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), POT_FULL,  POT_500MA, 0

 ; CC load - connect current meter to test point, verify ~1000 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), POT_FULL,  POT_1A,    0

 ; CV load - connect voltage meter to B+, verify == vEmpty (3V) [and current should be 500 mA]
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE),                    POT_500MA, 0,         0

 ; CC charge - connect current meter to test point, verify current is ~50 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), 0,         POT_FULL,  0

 ; CC charge - connect current meter to test point, verify current is ~100 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), POT_100MA, POT_FULL,  0

 ; CC charge - connect current meter to test point, verify current is ~500 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), POT_500MA, POT_FULL,  0

 ; CC charge - connect current meter to test point, verify current is ~1000 mA, voltage N/A
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)|MASK(_BAT_CHARGE_CC)|MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), POT_1A,    POT_FULL,  0

 ; CV charge - connect voltage meter to B+/B-, verify == vFull (4.2V) [and current should be 100 mA]
 db  MASK(_BAT_DIAGNOSTIC)|MASK(_BAT_CHARGE_ACTIVE)                     |MASK(_BAT_LOAD_ACTIVE)|MASK(_BAT_LOAD_CC), 0,         POT_100MA, 0

diagnostics_run_activate

    ; disallow if battery is present
    btfss   bat_flags, _BAT_LOADED
    bra     diag_run_go 
    SET_POPUP_MSG M_REMOVE_BAT
    bra     diagnostics_menu_activate

diag_run_go
    ; calc mode index from menu index
    movff   context_diagmenu, WREG
    decf    WREG,w
    movff   WREG, display_diag_setting
    ; set up operation request block
    LOADTBLPTR diagnostics_parm_table
    movff   display_diag_setting, WREG
    rlncf   WREG,w
    rlncf   WREG,w
    andlw   0xFC
    CALC_TABLE_ADDRESS
    tblrd*+
    movff   TABLAT, bat_request_data+0  ; 1'st parm is bat_flags
    CLR     bat_request_data+1, 2       ; 2'nd parm is target voltage (0 for CC modes)
    btfsc   bat_request_data+0, _BAT_CHARGE_CC
    bra     diag_run_not_CV_chg
    MOVLF16 DEF_BAT_FULL, bat_request_data+1
diag_run_not_CV_chg
    btfsc   bat_request_data+0, _BAT_LOAD_CC
    bra     diag_run_not_CV_load
    MOVLF16 DEF_BAT_EMPTY, bat_request_data+1
diag_run_not_CV_load
    call    operation_request
    ; set charge/load current values
    tblrd*+
    movff   TABLAT, iChg_filt
    tblrd*+
    movff   TABLAT, iLoad_filt
    setf    iLoad_limit
    ; show diagnostic screen
    LED_ON  LED_BLU
    SET_BASE_PAGE DIAGRUN_PAGENUM
    return

keyjumptab_runmenu
    ; send stop request
    call    operation_terminate
    ; prevent battery insert detection on diag exit
  bcf INTCON, GIEH
    CLR     bplus_filt, 2
    CLR     bplus_filt_err, 2
  bsf INTCON, GIEH
    ; return to menu
    LED_OFF LED_BLU
    bra     diagnostics_menu_activate


;----------------------------------------------------------------------
; Helpers
;----------------------------------------------------------------------

charge_start ; [simple charge or discharge]

    rcall   reset_statistics        ; reset energy & time counters
    rcall   calc_bat_timeout        ; set operation timeout

    ; record starting voltage for operation
    MOV16   bplus_filt, run_vStart

    ; start operation
    rcall   get_bplus_to_cnv_value  ; retrieve bplus[from interrupt] to cnv_value
    RR16    cnv_value, 1            ;   convert AD units to Vx100
    call    get_vFull               ; 2'nd parm = target voltage
    CP16    cnv_value, bat_request_data+1 ; CY if bplus <= vTarget 
    bc      start_charge_chg
    ;    discharge
    setf    run_type
    MOVLF   MASK(_BAT_LOAD_ACTIVE), bat_request_data+0 ; 1'st parm = bat_flags
    MOV16   bat_request_data+1, run_vEnd ; show the target [ending] voltage
    RL16    run_vEnd, 1                  ;   (V to ADC units)
    bra     start_charge_go
start_charge_chg
    ;    charge
    clrf    run_type
    MOVLF   MASK(_BAT_CHARGE_ACTIVE), bat_request_data+0 ; 1'st parm = bat_flags
    MOV16   bat_request_data+1, run_vEnd ; show the target [ending] voltage
    RL16    run_vEnd, 1                  ;   (V to ADC units)
start_charge_go
    call    operation_request

    ; operation started indication (RED LED on)
    LED_OFF LED_GRN
    LED_ON  LED_RED

    return

reset_statistics
    ; reset statistical accumulators
    clrf    log_index
    movlw   1                   ; start with 1 second/sample period
    movwf   log_period
    movwf   log_timer
    CLR     log_time,3
    ; reset power accumulators
    CLR     run_time,3
    CLR     run_accumulator,4
    return

;----------------------------------------------------------------------
; calculate appropriate operation timeout

calc_bat_timeout
    ; T[mins] = 2 * 60 * C / 1000 = C * 12 / 100
    ; approximated as C / 8
    MOV16   batcfg_C, bat_timeout
    RR16    bat_timeout, 3  ; /8
    ; init sub-timer
    MOVLF   60, bat_1min_timer
    return

;----------------------------------------------------------------------
; calculate charge/discharge current limits

limit_table ; limit*256
    dw   26
    dw   51
    dw   77
    dw  102
    dw  128
    dw  154
    dw  179
    dw  205
    dw  230
    dw  256
    dw  282
    dw  307
    dw  333
    dw  358
    dw  384
    dw  410
    dw  435
    dw  461
    dw  486
    dw  512

calc_limits

    ; no limits in diagnostic mode
    setf    iChg_limit
    setf    iLoad_limit
    btfsc   bat_flags, _BAT_DIAGNOSTIC
    return

    ; limit charge current to 1/2 C
    ;
    ;   I[mA] = 50 + POT * 3.90625  so  POT = (mA - 50) / 3.90625
    ;
    ;   POT (approximated) = (I/2-50)/4 + (I/2-50)/128

    MOV16   batcfg_C, fgnd_temp     ; C [mA]
    RR16    fgnd_temp, 1            ; calc C/2
    SUBL16  50, fgnd_temp           ; calc    -50
    RR16    fgnd_temp, 2            ; calc     /4
    MOV16   fgnd_temp, fgnd_temp+2  ; accumulating result into temp[2,3]
    RR16    fgnd_temp, 7            ; calc     /128
    ADD16   fgnd_temp, fgnd_temp+2

    movf    fgnd_temp+2, w          ; get result
    tstfsz  fgnd_temp+3             ;   overflowed 8 bits ?
    setf    WREG                    ;   yep, rail at 255
    movwf   iChg_limit

    ; limit discharge current per setting value
    ;
    ;   I[mA] = C * setting (where setting is 0.1 to 2.0)
    ;
    ;   POT (approximated) = (I/2-50)/4 + (I/2-50)/128  [same formula as for the charge limit]

    ; look up the multiplier
    LOADTBLPTR limit_table
    rlncf   batcfg_limit, w         ; 2 bytes/entry
    CALC_TABLE_ADDRESS
    tblrd*+
    movff   TABLAT, AARGB1
    tblrd*
    movff   TABLAT, AARGB0
    ; calculate I = C * setting
    movff   batcfg_C+0, BARGB1
    movff   batcfg_C+1, BARGB0
    call    FXM1616U                ; result in AARGB0,1,2,3
    ; result is X256, and it should be within 16 bits
    movff   AARGB2, fgnd_temp+0
    movff   AARGB1, fgnd_temp+1
    ; apply POT value approximation formula
    SUBL16  50, fgnd_temp           ; calc    -50
    RR16    fgnd_temp, 2            ; calc     /4
    MOV16   fgnd_temp, fgnd_temp+2  ; accumulating result into temp[2,3]
    RR16    fgnd_temp, 7            ; calc     /128
    ADD16   fgnd_temp, fgnd_temp+2
    movf    fgnd_temp+2, w          ; get result
    tstfsz  fgnd_temp+3             ;   overflowed 8 bits ?
    setf    WREG                    ;   yep, rail at 255
    movwf   iLoad_limit

    return

;----------------------------------------------------------------------
; compress log buffer to 1/2 size
;
;    it's lotza code, but quite fast :)
;
; [0] <-- [0]
; [1] <-- [2]
; [2] <-- [4]
; [3] <-- [6]
; [4] <-- [8]
; [5] <-- [10]
; ...
; [LOG_DISPLAY_WIDTH-1] <-- [(LOG_DISPLAY_WIDTH-1)*2]

KOMPRESS_LOG macro zLogAray
    variable i
i=1
  while i < LOG_DISPLAY_WIDTH
    movff   zLogAray+(i*2), zLogAray+i
i=i+1
  endw
    endm

do_log_compress ; compress both log buffers
    KOMPRESS_LOG log_V_aray 
    KOMPRESS_LOG log_I_aray
    return

;----------------------------------------------------------------------
; Increment time by 1 second

bump_time ; FSR0 points to SS field
    incf    INDF0,f    ; SS = SS+1
    movlw   59
    cpfsgt  INDF0
    bra     bump_time_done
    clrf    POSTDEC0
    incf    INDF0,f    ; MM = MM+1
    movlw   59
    cpfsgt  INDF0
    bra     bump_time_done
    clrf    POSTDEC0
    incf    INDF0,f    ; HH = HH+1
bump_time_done
    return
