;**********************************************************************
;                                   
; Li-Ion Battery Tool
;
; Copyright (C) 2010 Brute Force Coding, Ltd.
; BFC is a division of Really Silly Programming, Inc.
;
;**********************************************************************
;
; Based on CID-IV(2010)
; 09/2010 : [RSP] created (version number is in bat_pages.inc[about screen])
;
; Compiled with Microchip Assembler/Linker/etc v8.41              
; Target  : Microchip 18F2525/4525
;
; New EEPROM should be initialized to all 0xFF
;
;**********************************************************************

; NOTE - the program version number is in _pages.asm[ about ] screen data block

;-- compile control switches

; normally 1
#define ENABLE_EE_WRITES           1 ; EE i/o testing
; normally 0
#define ENABLE_BOOTIELOADER        0 ; vs. stubloader
#define ENABLE_BLINK_ON_PAGEUPDATE 0 ; screen update indicator
#define ENABLE_BLINK_ON_1SECTICK   0 ; timebase test

;-- CPU chip

#include    <p18f4525.inc>      ; processor specific variable definitions
#include    <pic18macros.inc>   ; general purpose helper macros

;#define LCD_CHIP EPSON     ; grn tab
#define LCD_CHIP PHILIPS    ; red tab

#include    "../bootload/chip_config.inc" ; application specific

;------------------------------------------------------------------------------
; Charset [16x6] special characters                            

CHAR_NOBREAKSPACE   EQU 0x80 ; overstrike space (numeric width)
CHAR_COPYRIGHT      EQU 0x81 ; copyright symbol

;**********************************************************************
;       D E F I N E S
;**********************************************************************

; screen page memory allocation
NBR_DYNAMIC_PAGES   EQU  17 ; must match nbr of dynamic pages in PAGE_LIST_TABLE (pages.inc)

; screen display timing
SPLASH_PERSIST      EQU 255 ; @ 50 ticks/sec = ~5 sec
POPUP_PERSIST       EQU  90 ; @ 50 ticks/sec = ~2 sec

; screen data auto-refresh rate
AUTOREPLOT_TICKS    EQU  5  ; @ 10 ticks/sec = 1/2 second

; input button timing
HOLDDOWN_TIME   EQU   3*50  ; TPA hold key time (1/50 secs == 3 seconds)
KEY_AUTOREPEAT_START EQU 50
KEY_AUTOREPEAT_SLOW  EQU 25 ; 50/sec divide-down
KEY_AUTOREPEAT_FAST  EQU 3
TPA_BOUNCES          EQU 2  ; consecutive keys @50/sec

; values for new EEPROM
;   display
SIZEOF_DISPCFG  EQU    2  ; size of this structure...
#if LCD_CHIP == EPSON
DEF_BACKLIGHT   EQU 255
DEF_CONTRAST    EQU 64      ; contrast = 25%
#endif
#if LCD_CHIP == PHILIPS
DEF_BACKLIGHT   EQU 255
DEF_CONTRAST    EQU 128     ; contrast = 50%
#endif
;  battery settings
SIZEOF_BATCFG   EQU    7  ; size of this structure..
DEF_BAT_C       EQU 2000    ; mAh
DEF_BAT_FULL    EQU  420    ; 4.2v [custom battery setting]
DEF_BAT_EMPTY   EQU  300    ; 3.0v [custom battery setting]
DEF_BAT_LIMIT   EQU    9    ; 1.0C discharge limit

; error codes
E_20ms_OVERRUN  EQU 1       ; foreground overrun (20 ms tick handler)
E_DISPLAY       EQU 2       ; bad page display data (invalid command)
E_ADC           EQU 3       ; ADC read sequence did not complete [in 2.5 msec]
E_ADC_SYNC      EQU 4       ; ADC state machine failed (bad channel)
E_TIMEOUT       EQU 5       ; operation timed out

; popup message codes
M_INSERTED      EQU 0       ; "Battery Inserted"
M_REMOVED       EQU 1       ; "Battery Removed"
M_CHARGING      EQU 2       ; "Charge Started"
M_REMOVE_BAT    EQU 3       ; "REMOVE BATTERY!"
M_CHARGE_DONE   EQU 4       ; "Charge Completed"
M_HEATSINK      EQU 5       ; heatsink overheated
M_OVERHEAT      EQU 6       ; battery overheated
M_NORESULTS     EQU 7       ; no results available (analyzer in precondition phase)
M_NOBATTERY     EQU 8       ; "Battery Not Present"
M_ANALYZING     EQU 9       ; "Analysis Started"

; charger/analyzer parameters
CHARGE_DWELL_TICKS EQU 50   ; charged when iLoad is minimal for this many 1/10 secs
DISCH_DWELL_TICKS  EQU 50   ; discharged when iLoad is zero for this many 1/10 secs
;   digital POT current settings
POT_100MA       EQU  13
POT_300MA       EQU  64     ; 3.92 mA / POT step, so 300 mA = 0.05A + 250 mA [64 POT steps]
POT_500MA       EQU 117
POT_1A          EQU 248
POT_FULL        EQU 255     ; ~1.05A

; standard battery voltages
;          Empty Full
;   STD     3.0  4.20
;   MIL     3.0  3.92
;   STORE   n/a  3.50
MIN_BAT_VOLTAGE  EQU 200    ; battery detection voltage
STD_BAT_EMPTY    EQU 300
STD_BAT_FULL     EQU 420
STD_BAT_MIL      EQU 392
STD_BAT_STORE    EQU 350    ; ~40% of STD_FULL
DEAD_BAT_VOLTAGE EQU 290
NO_BAT_VOLTAGE   EQU 1000   ; detect point for B+ when battery is removed during charging == 5v (nominal full scale)

; other battery parameters
MAX_CAPACITY       EQU 3000 ; mAh
MIN_CAPACITY       EQU  500 ; mAh
BAT_DEBOUNCE_TICKS EQU   40 ; 100 msec battery insert debounce

; thermal limits
FAN_ON_TEMP        EQU  120 ; F
FAN_OFF_TEMP       EQU  100 ; F
HEATSINK_OVERTEMP  EQU  180 ; F
BATTERY_OVERTEMP   EQU  120 ; F
FAN_ON_POT         EQU  128 ; POT units (~0.5A charge|load)
FAN_OFF_POT        EQU   64 ; POT units


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; EEPROM Memory
;
; PIC18F2525 has 1024 bytes of EEPROM
;
; Write rating is 10K minimum (per memory cell)
;----------------------------------------------------------------------
;----------------------------------------------------------------------

;   ORG 0xf00000

; block locations
EEADR_DISPLAY   EQU  0      ; LCD_... display settings
EEADR_BATTERY   EQU 64      ; battery settings

; all blocks must begin with a 2-byte signature (HI-LO)
;   so the actual EE storage length is data_length + 3
EE_SIGNATURE    EQU 0x3169


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; RAM data storage
;----------------------------------------------------------------------
;----------------------------------------------------------------------

#include    <bat_RAM.inc>


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Bootloader stub
;----------------------------------------------------------------------
;----------------------------------------------------------------------

#if ENABLE_BOOTIELOADER == 0

    ORG 0x000       ; RESET Vector
    bra POWERON

    ORG 0x008       ; Interrupt Vector
    bra     global_interrupt_hi
    
    ORG 0x018       ; Interrupt Vector
    bra     global_interrupt_lo

;-- main program run loop framework

    ORG 0x02A       ; user program return point from Start or Tick

    goto    user_program_tick   ; user must return to BOOT_RUN (== 0x002A)

;-- simplified bootloader startup

POWERON ; power-on reset

    clrf    INTCON          ; insure interrupts are off

;-- set up I/O ports

    clrf    WREG                ; normal initialization fcn code
    rcall   user_program_config ; application specific port init, & turns on diag LED

;-- configure comms for 9600 rate 

    bsf TXSTA,   TXEN
    bsf RCSTA,   SPEN
    bsf RCSTA,   CREN

    bsf BAUDCON, BRG16
    MOVLF   high(BAUD_9600), SPBRGH
    MOVLF    low(BAUD_9600), SPBRG

;-- detect bad power-up

    movf    RCON,w
    ; clear error conditions (allow MCLR to reset the CPU to normal operation)
    bsf     RCON, RI        ; /RI is writable directly
    clrwdt                  ; /TO is not writable directly; but clrwdt resets it

    btfss   WREG, RI
    bra     is_reset_instr
    btfsc   WREG, TO
    bra     not_WDT

is_watchdog     ; 1 blink == watchdog timeout
    movlw   1
    bra     hw_fault_handler

is_reset_instr
    movlw   2   ; 2 blinks == reset instruction

hw_fault_handler

; at this point foregnd+interrupt have died, so take over RAM bank 0 for fault handling vars
#define err_code      0
#define blink_counter 1
#define delay_timer_0 2
#define delay_timer_1 3

    movwf   err_code        ; save error code (blink count)

hw_fault_bigloop ; must cycle power to reset a hardware fault

    movff   err_code, blink_counter
hw_fault_blink
    bcf     DIAG_LED        ; LED on
    rcall   blink_delay
    bsf     DIAG_LED        ; LED off
    rcall   blink_delay
    decfsz  blink_counter,f
    bra     hw_fault_blink
    ; delay between sets of blinks
    rcall   blink_delay
    rcall   blink_delay
    rcall   blink_delay
    rcall   blink_delay
    bra     hw_fault_bigloop

blink_delay ; nominal 1/4 second delay [@ 10 MHz fCyc]
    MOVLF     8, delay_timer_0
blink_delay_loop_31250us
    MOVLF   250, delay_timer_1
blink_delay_loop_125us
    movlw   250
blink_delay_loop_500ns              ; each cyc == 100 nsec (40MHz fOsc = 10MHz fCyc)
    nop                             ; 1 cyc
    nop                             ; 1 cyc
    decfsz  WREG,w                  ; 1 cyc
    bra     blink_delay_loop_500ns  ; 2 cyc
    decfsz  delay_timer_1, f
    bra     blink_delay_loop_125us
    decfsz  delay_timer_0, f
    bra     blink_delay_loop_31250us
    return

not_WDT

;-- auto-start user program

    bra     user_program_start  ; user must return to BOOT_RUN (== 0x002A)

#endif


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Start of User Program Area - Vectors
;----------------------------------------------------------------------
;----------------------------------------------------------------------

    ORG 0x02A       ; bootloader
BOOT_RETURN         ;   return point for startup/tick

#define APP_ORG 0x340           ; must match bootloader!

	ORG	APP_ORG
user_program_config ; user program config (see below for WREG parm values for this function)
    bra     program_config      ; local (below), called by bootloader only

	ORG	APP_ORG+0x04
user_program_start  ; user program start 
    bra     STARTUP             ; in _startup.inc
;   goto    STARTUP             ; in _startup.inc

	ORG	APP_ORG+0x08
global_interrupt_lo ; global interrupt
    bra     LOW_PRIORITY_INT    ; in _interrupt.inc
;   goto    LOW_PRIORITY_INT    ; in _interrupt.inc

	ORG	APP_ORG+0x0C
global_interrupt_hi ; global interrupt
    bra     HIGH_PRIORITY_INT   ; in _interrupt.inc
;   goto    HIGH_PRIORITY_INT   ; in _interrupt.inc

	ORG	APP_ORG+0x10
user_program_tick
    bra     foreground_tick
;   goto    foreground_tick


program_config   ; WREG=1/2 for diag LED off/on; else normal initialization

    ; program_config vector is overloaded so that it can also control the diag LED
    ;   (bootloader needs to control this LED, but it must be an app reconfigurable I/O port too)
    dcfsnz  WREG,w
    bra     config_diag_LED_off
    dcfsnz  WREG,w
    bra     config_diag_LED_on
    ; normal initialization
    INIT_PORTS
    return

    DIAG_LED_CONTROL ; actual LED control code (config_diag_LED_on/off routines)

;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Supporting Code
;----------------------------------------------------------------------
;----------------------------------------------------------------------

; startup initialization

#include    <bat_startup.inc>    ; startup

; interrupt handler

#include    <bat_interrupt.inc>  ; interrupt handler
#include    <bat_analyzer.inc>   ; battery charger/analyzer

; main program

#include    <bat_foreground.inc> ; main program
#include    <bat_cnv.inc>        ; display formatting subroutines

; libraries

#include    <bat_math.inc>       ; 31/23 divide and 16/16 multiply routiens

; hardware drivers

#include    <bat_digpot.inc>     ; digital pot i/o library
#include    <bat_LCD.inc>        ; Nokia 6100 Color LCD driver
#include    <bat_EE.inc>         ; SPI EEROM interface

; data tables

#include    <bat_pages.inc>      ; application screens

#include    <ntc_20K.inc>        ; NTC conversion table

#include    <bat_graph.inc>      ; results graph screen conversion tables

#include    <font16.inc>         ; normal text font
#include    <font24.inc>         ; large text font
FONT_TABLE
FNT_16x6      EQU 0*2*3
    dw  16, CHARSET_16_INDEX,    CHARSET_16_SIZES      ; 0
FNT_24x10     EQU 1*2*3
    dw  24, CHARSET_24_INDEX,    CHARSET_24_SIZES      ; 1

;----------------------------------------------------------------------
;----------------------------------------------------------------------
    END
;----------------------------------------------------------------------
;----------------------------------------------------------------------
